--[[
NOTICE: This is a copy of the file in the Elinor mod by lawrence (the pikeman guy).
That includes commentary. I repeat, not always my opinion.

It's modified by Mobbstar to work properly with the new DS versions.

Receives the name of the new tech branch and its max level. The max level
is used to create entries in TECH. If, say, name == "SHENANIGANS", these
entries have the form TECH.SHENANIGANS_1, TECH_SHENANIGANS_2, ... with
the number going up to maxlevel.

For uniformity with the game's own tech trees, it's best to use an
uppercase name.
]]
function AddTechBranch(name, maxlevel) --e.g. AddTechBranch("BIOLOGY",2)
	-- Inserts the new branch in the TECH predef trees.
	for _, v in pairs(GLOBAL.TECH) do
		v[name] = v[name] or 0
	end

	-- The same, but for the prototyper trees.
	for _, v in pairs(GLOBAL.TUNING.PROTOTYPER_TREES) do
		v[name] = v[name] or 0
	end

	-- And now we create our own predefs.
	--Mobbstar here, I have no idea why that is
	--I need to look at TECH NONE first I guess
	--I think it means that the tech tree is called SOUL_1, etc.
	for i = 1, (maxlevel or 1) do
		local newtech = {[name] = i}
		for k, v in pairs(GLOBAL.TECH.NONE) do
			newtech[k] = newtech[k] or v
			
		end
		GLOBAL.TECH[("%s_%d"):format(name, i)] = newtech
	end

	--[[
	The following is needed to reset the custom tech level when
	leaving proximity. It is not needed for vanilla branches,
	but for custom ones it is (see Builder:EvaluateTechTrees() for
	details).
	
	An alternative would be to do this resetting through the onturnoff
	callback of the Prototyper component, like Heavenfall does, but that
	requires assuming that the only builder entity in the game is the
	player, so I prefer this method. Neither of them are clean enough
	by my judgement, though: the Builder component has far too much
	hardcoded logic.
	]]
	
	local bonusname = string.lower(name) .."_bonus" --to be in-line with the game
	
	local Builder = GLOBAL.require "components/builder"
	Builder[bonusname] = Builder[bonusname] or 0
	
	Builder.EvaluateTechTrees = (function()
		local oldEval = Builder.EvaluateTechTrees

		return function(self)
			local had_prototyper = self.current_prototyper
			
			oldEval(self)
			
			if had_prototyper and not self.current_prototyper then
				-- If there is a new prototyper, it'll take care of
				-- setting the values right anyways.
				self.accessible_tech_trees[name] = 0
				self.inst:PushEvent("techtreechange", {level = self.accessile_tech_trees})
			end
		end
	end)() --didn't know this was possible!
	
	Builder.KnowsRecipe = (function() --done by me, mobbstar
		local oldKnows = Builder.KnowsRecipe

		return function(self,recname)
			local recipe = GLOBAL.GetRecipe(recname)

			if recipe and recipe.level[name] and recipe.level[name] > Builder[bonusname]
			and not self.freebuildmode and not table.contains(self.recipes, recname) then
				return false
			end
			
			return oldKnows(self,recname)
		end
	end)()
end
