local unpack = GLOBAL.unpack
local require = GLOBAL.require
local GetPlayer = GLOBAL.GetPlayer
local GetClock = GLOBAL.GetClock
local GetString = GLOBAL.GetString
local Vector3 = GLOBAL.Vector3
local Ents = GLOBAL.Ents
local SpawnPrefab = GLOBAL.SpawnPrefab
local saPrint = GLOBAL.saPrint
local PI = GLOBAL.PI
local FindWalkableOffset = GLOBAL.FindWalkableOffset
local ANCHOR_MIDDLE = GLOBAL.ANCHOR_MIDDLE
local EQUIPSLOTS = GLOBAL.EQUIPSLOTS
local RoG = GLOBAL.IsDLCEnabled(GLOBAL.REIGN_OF_GIANTS)
local IsSoulless = GLOBAL.IsSoulless
local IsSentient = GLOBAL.IsSentient

require "class"
require "actions"
local Image = require "widgets/image"
local IceOver = require "widgets/iceover"
--local HeatOver = require "widgets/heatover" --apparently I can't import heatover at all...

--[[ List of Effects (functions below)
Heal, Damage, Regen, Poison, Acid, Fire, Ice, Temperature, RunSpeed, CombatStrength, Insanity,
Sleep, Light, Shield, Blind, Invisible, FireImmune, IceImmune, GrueImmune, Blink, Slip,
Explode, Colour, Summon, Thorns, Soul;
]]

GLOBAL.POTIONEFFECTS = {}
GLOBAL.POTIONAFFINITY = {}
GLOBAL.POTIONWEAROFF = {}
GLOBAL.POTIONBASED = {}
local POTIONEFFECTS = GLOBAL.POTIONEFFECTS
local POTIONAFFINITY = GLOBAL.POTIONAFFINITY
local POTIONWEAROFF = GLOBAL.POTIONWEAROFF
local POTIONBASED = GLOBAL.POTIONBASED

--the following two functions serve to avoid mult-colour issues (hue and transparency)
--TODO check what happens when a forced effect runs out first
local function RemoveColourer(inst,eff)
	for k,othereff in pairs(inst.colourfx) do
		if othereff == eff then
			table.remove(inst.colourfx,k)
		end
	end
	
	if #inst.colourfx == 0 and inst.AnimState then
		if inst.inheritcolour then
			inst.AnimState:SetMultColour(unpack(inst.inheritcolour))
		else
			inst.AnimState:SetMultColour(1,1,1,1)
		end
	end
end
local function AddColourer(inst,eff,colour,force)
	if inst.colourfx then
		table.insert(inst.colourfx,eff)
		if force then
			inst.AnimState:SetMultColour(unpack(colour))
		end
	else
		inst.inheritcolour = {inst.AnimState:GetMultColour()}
		inst.AnimState:SetMultColour(unpack(colour))
		inst.colourfx = {eff}
	end
end

--Healing increases health instantly
POTIONEFFECTS.Heal = function(doer,args)
	if doer and doer.components.health and not (doer:HasTag("wall") or doer:HasTag("mech")) then
		doer.components.health:DoDelta(args.val,nil,"alchemypotion")
	end
end
POTIONAFFINITY.Heal = {"Damage"}

--Damage decreases health instantly
POTIONEFFECTS.Damage = function(doer,args)
	if doer and doer.components.health and not (doer:HasTag("wall") or doer:HasTag("mech")) then
		doer.components.health:DoDelta(args.val,nil,"alchemypotion")
	end
end
POTIONAFFINITY.Damage = {"Heal"}

--Regen periodically increases health (you can define the tick length too)
POTIONEFFECTS.Regen = function(doer,args)
	if doer and doer.components.health and not (doer:HasTag("wall") or doer:HasTag("mech")) then
		doer.healpotiontask = doer:DoPeriodicTask(args.rate or 1,function()
			doer.components.health:DoDelta(args.val or 1,true,"alchemypotion")
		end)
		if doer.HUD then
			local anim = "arrow_loop_increase_more"
			if args.rate and args.val and (args.val/args.rate > 3 or args.rate <= .5)
				and not doer.HUD.controls.status.heart.arrowdir == "arrow_loop_decrease_most" then
				anim = "arrow_loop_increase_most"
			end
			doer.HUD.controls.status.heart.sanityarrow:GetAnimState():PlayAnimation(anim, true)
		end
		return true
	end
end
POTIONAFFINITY.Regen = {"Poison"}
POTIONWEAROFF.Regen = function(doer,args)
	if doer and doer.healpotiontask then
		doer.healpotiontask:Cancel()
		doer.healpotiontask = nil
	end
	if doer.HUD then
		doer.HUD.controls.status.heart.sanityarrow:GetAnimState():PlayAnimation(doer.HUD.controls.status.heart.arrowdir, true)
	end
end

--Poison periodically decreases health (you can define the tick length too)
POTIONEFFECTS.Poison = function(doer,args)
	if doer and doer.components.health and not (doer:HasTag("wall") or doer:HasTag("mech")) then
		doer.poisontask = doer:DoPeriodicTask(args.rate or 1,function()
			doer.components.health:DoDelta(args.val or -1,true,"alchemypotion")
		end)
		if doer.HUD then
			local anim = "arrow_loop_decrease_more"
			if args.rate and args.val and (args.val/args.rate > 3 or args.rate <= .5)
				or doer.HUD.controls.status.heart.arrowdir == "arrow_loop_decrease_most" then
				anim = "arrow_loop_decrease_most"
			end
			doer.HUD.controls.status.heart.sanityarrow:GetAnimState():PlayAnimation(anim, true)
		end
		return true
	end
end
POTIONAFFINITY.Poison = {"Regen"}
POTIONWEAROFF.Poison = function(doer,args)
	if doer and doer.poisontask then
		doer.poisontask:Cancel()
		doer.poisontask = nil
	end
	if doer.HUD then
		doer.HUD.controls.status.heart.sanityarrow:GetAnimState():PlayAnimation(doer.HUD.controls.status.heart.arrowdir, true)
	end
end

--Acid is poison, but doesn't check for expection tags
POTIONEFFECTS.Acid = function(doer,args)
	if doer and doer.components.health then
		if doer.SoundEmitter then
			doer.SoundEmitter:PlaySound(RoG and "dontstarve_DLC001/common/HUD_hot_level3" or "dontstarve/wilson/burned")
		end
		doer.acidtask = doer:DoPeriodicTask(args.rate or 1,function()
			doer.components.health:DoDelta(args.val or -1,true,"alchemypotion")
		end)
		if doer.HUD then
			local anim = "arrow_loop_decrease_more"
			if args.rate and args.val and (args.val/args.rate > 3 or args.rate <= .5)
				or doer.HUD.controls.status.heart.arrowdir == "arrow_loop_decrease_most" then
				anim = "arrow_loop_decrease_most"
			end
			doer.HUD.controls.status.heart.sanityarrow:GetAnimState():PlayAnimation(anim, true)
		end
		return true
	end
end
POTIONWEAROFF.Acid = function(doer,args)
	if doer and doer.acidtask then
		doer.acidtask:Cancel()
		doer.acidtask = nil
	end
	if doer.HUD then
		doer.HUD.controls.status.heart.sanityarrow:GetAnimState():PlayAnimation(doer.HUD.controls.status.heart.arrowdir, true)
	end
end

--Fire ignites the target if possible (and if not :P)
local function putout(fx)
	fx.components.firefx:Extinguish()
	if fx.dmgfn then fx.dmgfn:Cancel() end --cease damage
	fx:ListenForEvent("animover", function(fx) fx:Remove() end)
	fx.guy:RemoveTag("fire")
end
local function firedmg(fx)
	if fx.nodelay and fx.guy and fx.guy.components.health then
		fx.guy.components.health:DoFireDamage(5,fx)
	else
		fx.nodelay = true --stay somewhat in sync with normal fire mechanics
	end
end
POTIONEFFECTS.Fire = function(doer)
	if doer then
		if doer.components.burnable then
			doer.components.burnable:Ignite()
		elseif doer.components.health and not doer:HasTag("fire") then
			--visual
			local fx = SpawnPrefab("character_fire")
			if fx then
				local pt = doer:GetPosition()
				doer:AddTag("fire")
				fx.guy = doer
				fx.Transform:SetPosition(pt.x,pt.y,pt.z)
				fx.components.firefx.playignitesound = false
				fx.components.firefx:SetLevel(2)
				fx:DoTaskInTime(8,putout) --extinguish
				fx.dmgfn = fx:DoPeriodicTask(.5,firedmg) --do damage
				fx.entity:AddFollower():FollowSymbol( doer.GUID, "torso", 0,0,.1)
			end
		end
	end
end
POTIONAFFINITY.Fire = {"Ice","FireImmune"}

--Ice freezes the target if possible
POTIONEFFECTS.Ice = function(doer,args)
	if doer and doer.components.freezable then
		doer.components.freezable:AddColdness(args.val)
	end
end
POTIONAFFINITY.Ice = {"Fire","IceImmune"}

--Temperature changes the temperature of the target if possible
POTIONEFFECTS.Temperature = function(doer,args)
	if doer and doer.components.temperature then
		doer.components.temperature:SetTemperature(doer.components.temperature.current + args.val)
		if doer.HUD then
			if args.val > 0 and RoG then
				--[[local hud = HeatOver()
				hud.alpha_min_target = .6
				hud.effectSize_target = 0.01
				hud.effectFrequency_target = 8
				hud:StartUpdating()
				hud.inst:DoTaskInTime(.6,function() hud.alpha_min_target = 1 end)
				hud.inst:DoTaskInTime(1,function() hud:Kill() end)]]
				doer.HUD.fireover:TurnOn()
				doer:DoTaskInTime(.6,function() doer.HUD.fireover:TurnOff() end)
			else
				local hud = IceOver()
				hud.alpha_min_target = .6
				hud:StartUpdating()
				hud.inst:DoTaskInTime(.6,function() hud.alpha_min_target = 1 end)
				hud.inst:DoTaskInTime(1,function() hud:Kill() end)
			end
		end
	end
end

--RunSpeed modifies the movement speed of the target, if possible
POTIONEFFECTS.RunSpeed = function(doer,args)
	if doer and doer.components.locomotor then
		if doer.components.locomotor.walkspeed then
			doer.components.locomotor.walkspeed = doer.components.locomotor.walkspeed + args.val
		end
		if doer.components.locomotor.runspeed then
			doer.components.locomotor.runspeed = doer.components.locomotor.runspeed + args.val
		end
		return true
	end
end
POTIONWEAROFF.RunSpeed = function(doer,args)
	if doer and doer.components.locomotor then
		if doer.components.locomotor.walkspeed then
			doer.components.locomotor.walkspeed = doer.components.locomotor.walkspeed - args.val
		end
		if doer.components.locomotor.runspeed then
			doer.components.locomotor.runspeed = doer.components.locomotor.runspeed - args.val
		end
	end
end

--CombatStrength changes the damage output of the target
POTIONEFFECTS.CombatStrength = function(doer,args)
	if doer and doer.components.combat then
		if doer.components.combat.damagebonus then
			doer.components.combat.damagebonus = doer.components.combat.damagebonus + args.val
		else
			doer.components.combat.damagebonus = args.val
		end
		if doer.SoundEmitter then
			if args.val > 0 then
				doer:DoTaskInTime(1,doer.SoundEmitter:PlaySound("dontstarve/creatures/cavespider/scream"))
			else
				doer:DoTaskInTime(1,doer.SoundEmitter:PlaySound("dontstarve/creatures/cavespider/die"))
			end
		end
		return true
	end
end
POTIONWEAROFF.CombatStrength = function(doer,args)
	if doer and doer.components.combat then
		doer.components.combat.damagebonus = doer.components.combat.damagebonus - args.val
		if doer.components.combat.damagebonus == 0 then
			doer.components.combat.damagebonus = nil
		end
	end
end

--Insanity induces total insanity for a fixed time (giving the amulet priority)
POTIONEFFECTS.Insanity = function(doer,args)
	if doer and doer.components.sanity and not doer.components.sanity.inducedinsanity then
		doer.components.sanity.inducedinsanity = "POTION" --so we can tell whether it's ours
		return true
	end
end
POTIONWEAROFF.Insanity = function(doer,args)
	if doer and doer.components.sanity.inducedinsanity == "POTION" then
		doer.components.sanity.inducedinsanity = nil
	end
end

--Sleep puts the target to sleep if possible
POTIONEFFECTS.Sleep = function(doer,args)
	if doer then
		if args.val > 0 and doer:HasTag("player") then --hack, because the player doesn't have sleeper
			TheFrontEnd:Fade(false,.2)
			doer:DoTaskInTime(0.3, function() 
				doer.sg:GoToState("wakeup")
				--doer.components.talker:Say(GetString("mandrake", "ANNOUNCE_KNOCKEDOUT"))
				TheFrontEnd:Fade(true,.8)
				GetClock():MakeNextDay()
			end)
		elseif doer.components.sleeper then
			doer.components.sleeper:AddSleepiness(args.val)
		end
	end
end

--Light attaches a light to the target
POTIONEFFECTS.Light = function(doer,args)
	if doer then
		if doer.wormlight then
			doer.wormlight.components.spell.duration = math.max(args.dur,
				doer.wormlight.components.spell.duration - doer.wormlight.components.spell.lifetime)
			doer.wormlight.components.spell.lifetime = 0
			doer.wormlight.components.spell:ResumeSpell()
		else
			local light = SpawnPrefab("wormlight_light")
			light.components.spell:SetTarget(doer)
			light.components.spell.duration = args.dur
			if not light.components.spell.target then
				light:Remove()
			end
			light.components.spell:StartSpell()
		end
	end
end

--Shield adds temporary resistance to the target
POTIONEFFECTS.Shield = function(doer,args)
	if doer and doer.components.health then
		doer.components.health.absorb = doer.components.health.absorb + args.val --in percent!
		if not doer.shieldfx then
			doer.shieldfx = SpawnPrefab("shieldfx")
			doer.shieldfx.AnimState:SetMultColour(1,1,.4,.5)
			doer.shieldfx.owner = doer
			doer.shieldfx:SpawnShields(args.shields or 2)
		end
		return true
	end
end
POTIONWEAROFF.Shield = function(doer,args)
	if doer then
		if doer.components.health then
			doer.components.health.absorb = doer.components.health.absorb - args.val
		end
		if doer.shieldfx then
			doer.shieldfx:OnRemove()
			doer.shieldfx = nil
		end
	end
end

--Blind disables brains or creates a big widget to cover the screen
POTIONEFFECTS.Blind = function(doer,args)
	if doer then
		if doer.HUD and doer.HUD.blindover then
			doer.HUD.blindover.inst.components.fader:StopAll()
			
			table.insert(doer.blindnessalpha, args.alpha)
			table.sort(doer.blindnessalpha)
			
			doer.HUD.blindover:SetTint(0,0,0,doer.blindnessalpha[#doer.blindnessalpha])
			doer.HUD.blindover:Show()
		elseif doer.brain then
			doer:StopBrain()
			SpawnPrefab("statue_transition_2").Transform:SetPosition(doer:GetPosition():Get())
		end
		return true
	end
end
POTIONWEAROFF.Blind = function(doer,args)
	if doer then
		if doer.HUD and doer.HUD.blindover then
			for k,v in ipairs(doer.blindnessalpha) do
				if v == args.alpha then
					table.remove(doer.blindnessalpha,k)
					break
				end
			end
			if #doer.blindnessalpha == 0 then
				doer.HUD.blindover.inst.components.fader:Fade(args.alpha, 0, args.fadetime or 1, function(val,inst)
					inst.widget:SetTint(0,0,0,val)
				end,function(inst) inst.widget:Hide() end)
			elseif doer.blindnessalpha[#doer.blindnessalpha] < args.alpha then
				doer.HUD.blindover.inst.components.fader:Fade(args.alpha, doer.blindnessalpha[#doer.blindnessalpha], args.fadetime or 1, function(val,inst)
					inst.widget:SetTint(0,0,0,val)
				end,function(inst) end)
			end
			
			--[[doer.potionblind.inst:AddComponent("fader")
			doer.potionblind.inst.components.fader:Fade(args.alpha,0,args.fadetime or 1,function(val,inst)
				inst.widget:SetTint(0,0,0,val)
			end,function(inst) inst.widget:Kill() end)]]
		else
			doer:RestartBrain()
		end
	end
end

--Hover slightly raises the target and has it gently move up and down (Doesn't work, don't use!)
POTIONEFFECTS.Hover = function(doer,args)
	if doer and doer.Physics and not doer:HasTag("flying") then
		doer:AddTag("flying")
		
		doer.hovertask = doer:DoPeriodicTask(.5,function()
			local pt = doer:GetPosition()
			local vel = Vector3(doer.Physics:GetMotorVel())
			
			if doer.hoverdown then
				saPrint("Should go down")
				doer.Physics:SetMotorVel(vel.x,-1,vel.z)
				if pt.y < .5 then
					doer.hoverdown = false
				end
			else
				saPrint("Should go up")
				doer.Physics:SetMotorVel(vel.x,1,vel.z)
				if pt.y > .8 then
					doer.hoverdown = true
				end
			end
		end)
		return true
	end
end
POTIONWEAROFF.Hover = function(doer,args)
	if doer and doer.hovertask then
		doer.hovertask:Cancel()
		doer:RemoveTag("flying")
	end
end

--Invisible makes the target invisible and hides it from other creatures
local function makehidden(inst)
	inst.sg:AddStateTag("invisible") --apparently that works?
	inst:AddTag("notarget")
end
POTIONEFFECTS.Invisible = function(doer,args)
	if not doer.invislevel then
		if doer and doer.sg then
			doer:ListenForEvent("newstate",makehidden)
		end
		
		doer.invislevel = 0
	end
	doer.invislevel = doer.invislevel + 1
	
	--visuals
	if not args.visible then
		doer:Hide()
	end
	
	return true
end
POTIONWEAROFF.Invisible = function(doer,args)
	if doer and doer.sg then
		doer:RemoveEventCallback("newstate",makehidden)
		doer:RemoveTag("notarget")
	end
	
	if doer.invislevel then
		doer.invislevel = doer.invislevel - 1
		if doer.invislevel == 0 then
			doer.invislevel = nil
			if not args.visible then
				doer:Show()
			end
		end
	elseif not args.visible then
		doer:Show()
	end
end

--FireImmune makes the target immune to fire damage
POTIONEFFECTS.FireImmune = function(doer,args)
	if doer then
		if doer.components.health then
			doer.nofirefj = doer.components.health.vulnerabletoheatdamage
			doer.components.health.vulnerabletoheatdamage = false
		end
		if doer.components.burnable and doer.components.propagator then
			doer.oldburnfn = doer.components.propagator.onflashpoint
			doer.components.propagator:SetOnFlashPoint(function() end)
		end
		if not doer.fireshield then
			doer.fireshield = SpawnPrefab("shieldfx")
			doer.fireshield.AnimState:SetMultColour(1,.2,.2,.5)
			doer.fireshield.height = 1.3
			doer.fireshield.reverse = true
			doer.fireshield.owner = doer
			doer.fireshield:SpawnShields(args.shields or 2)
		end
		return true
	end
end
POTIONWEAROFF.FireImmune = function(doer,args)
	if doer then
		if doer.components.health then
			doer.components.health.vulnerabletoheatdamage = doer.nofirefj
		end
		if doer.components.burnable and doer.components.propagator then
			doer.components.propagator:SetOnFlashPoint(doer.oldburnfn)
		end
		if doer.fireshield then
			doer.fireshield:OnRemove()
			doer.fireshield = nil
		end
	end
end

--IceImmune makes the target unfreezable
local function unfreeze(inst)
	inst:DoTaskInTime(.01,function(inst)
		inst.components.freezable:Thaw(0)
		--inst.components.freezable:Unfreeze()
	end)
end
POTIONEFFECTS.IceImmune = function(doer,args)
	if doer and doer.components.freezable then
		doer:ListenForEvent("freeze",unfreeze)
		if not doer.iceshield then
			doer.iceshield = SpawnPrefab("shieldfx")
			doer.iceshield.AnimState:SetMultColour(.4,.5,1,.5)
			doer.iceshield.height = 1.7
			doer.iceshield.owner = doer
			doer.iceshield:SpawnShields(args.shields or 2)
		end
		return true
	end
end
POTIONWEAROFF.IceImmune = function(doer,args)
	if doer then
		doer:RemoveEventCallback("freeze",unfreeze)
		if doer.iceshield then
			doer.iceshield:OnRemove()
			doer.iceshield = nil
		end
	end
end

--SleepImmune makes the target unable to sleep (unsolvable so far)
--Impossible for the player, due to how madhack mandrakes are

--GrueImmune makes the player (pseudo-)invincible to the nightmonster
local function stopgrue(inst)
	if inst and inst.components.grue then
		inst.components.grue.nextHitTime = 255
		inst.components.grue.nextSoundTime = 255
	end
end
POTIONEFFECTS.GrueImmune = function(doer,args)
	if doer and doer.components.grue then
	--Removing the component did nothing, presumably because the event
	--listeners carry on existing. Poor programming on Klei's side.
		doer:ListenForEvent("enterdark",stopgrue)
		stopgrue(doer)
		if not doer.grueshield then
			doer.grueshield = SpawnPrefab("shieldfx")
			doer.grueshield.AnimState:SetMultColour(.1,.1,.1,.5)
			doer.grueshield.height = .6
			doer.grueshield.reverse = true
			doer.grueshield.owner = doer
			doer.grueshield:SpawnShields(args.shields or 2)
		end
		return true
	end
end
POTIONAFFINITY.GrueImmune = {"Blind"}
POTIONWEAROFF.GrueImmune = function(doer,args)
	if doer then
		doer:RemoveEventCallback("enterdark",stopgrue)
		if doer.grueshield then
			doer.grueshield:OnRemove()
			doer.grueshield = nil
		end
	end
end

--Blink teleports the target somewhere within close proximity
POTIONEFFECTS.Blink = function(doer,args)
	if doer and doer.Transform then
		--args = args or {}
		local pt = doer:GetPosition()
		local offset = FindWalkableOffset(pt, math.random() * 2 * PI, math.random(args.minr or 5,args.maxr or 12), 12)
		if offset then
			pt = pt + offset
			doer.Transform:SetPosition(pt:Get())
		end
	end
end

--Slip makes the target drop their active item, or the first slot
POTIONEFFECTS.Slip = function(doer,args)
	if doer and doer.components.inventory then
		if doer.components.inventory.activeitem then --ideally, drop the held item
			doer.components.inventory:DropActiveItem()
		elseif doer.components.inventory.equipslots[EQUIPSLOTS.HANDS] then --or the handslot item
			doer.components.inventory:DropItem(doer.components.inventory.equipslots[EQUIPSLOTS.HANDS])
		elseif doer.components.inventory.equipslots[EQUIPSLOTS.HEAD] then --or blow the armor off
			doer.components.inventory:DropItem(doer.components.inventory.equipslots[EQUIPSLOTS.HEAD])
		else --just drop something!
			doer.components.inventory:DropItem(doer.components.inventory:FindItem(function() return true end))
		end
	end
end

--Explode makes the target pseudo-explode, either instantly or after some time
POTIONBASED.Explode = true
POTIONEFFECTS.Explode = function(doer,args,src)
	if not args.instant then
		return true
	end
	if doer then
		if doer.components.explosive then
			doer.components.explosive:OnBurnt() -- ...or our target is explosive already
		else
			if src then
				src.SoundEmitter:PlaySound("dontstarve/common/blackpowder_explo")
				local pt = src:GetPosition()
				local fx = SpawnPrefab("explode_small")
				fx.Transform:SetPosition(pt.x, pt.y, pt.z)
				fx.AnimState:SetBloomEffectHandle( "shaders/anim.ksh" )
				fx.AnimState:SetLightOverride(1)
				
				GetClock():DoLightningLighting()
				GetPlayer().components.playercontroller:ShakeCamera(src, "FULL", 0.7, 0.02, .5, 40)
			end
			
			if doer.components.workable and not doer:HasTag("busy") then --Haaaaaaack!
				doer.components.workable:WorkedBy(doer,6)
			elseif args.fire and doer.components.burnable and not doer.components.fueled then
			   doer.components.burnable:Ignite()
			end
			
			if doer.components.combat then
				doer.components.combat:GetAttacked(src or doer, -args.dmg)
			elseif doer.components.health then
				doer.components.health:DoDelta(-args.dmg)
			end
		end
	end
end
POTIONWEAROFF.Explode = function(doer,args,src)
	--if args.instant then
		--return true
	--end
	if not src then
		src = doer
	end
	
	if src then
		if src.components.explosive then
			src.components.explosive:OnBurnt() -- ...or our target is explosive already
		else
			src.SoundEmitter:PlaySound("dontstarve/common/blackpowder_explo")
			local pt = src:GetPosition()
			local fx = SpawnPrefab("explode_small")
			fx.Transform:SetPosition(pt.x, pt.y, pt.z)
			fx.AnimState:SetBloomEffectHandle( "shaders/anim.ksh" )
			fx.AnimState:SetLightOverride(1)
			
			GetClock():DoLightningLighting()
			GetPlayer().components.playercontroller:ShakeCamera(src, "FULL", 0.7, 0.02, .5, 40)
			
			if doer.components.workable and not doer:HasTag("busy") then --Haaaaaaack!
				doer.components.workable:WorkedBy(doer,6)
			elseif args.fire and doer.components.burnable and not doer.components.fueled then
			   doer.components.burnable:Ignite()
			end
			
			--[[if doer.components.combat then
				doer.components.combat:GetAttacked(src, -args.dmg)
			else]]if doer.components.health then
				doer.components.health:DoDelta(-args.dmg)
			end
		end
	end
end

--meat tag?

--Rescale changes the target's scale (unsolvable so far)
--How to remember the original scale?

--Colour changes the target's tint
POTIONEFFECTS.Colour = function(doer,args)
	if doer and doer.AnimState then
		AddColourer(doer,args,{args.r or 1,args.g or 1,args.b or 1,args.a or 1},args.force)
		return true
	end
end
POTIONWEAROFF.Colour = function(doer,args)
	if doer and doer.AnimState then
		RemoveColourer(doer,args)
	end
end

--Ghost changes the physics of the target (unsolvable so far)
--how to tell what the original masks were? Can't disable collision due to ground & occean
--[[POTIONEFFECTS.Ghost = function(doer,args)
	if doer and doer.Physics then
		args.oldmass = doer.Physics:GetMass()
		doer.Physics:SetMass(0) --prevent falling through world?
		doer.Physics:SetCollides(false) --no collisions whatsoever (border?)
		--remember to store the original values
		--clear mask
		--set ghost mask
		return true
	end
end
POTIONWEAROFF.Ghost = function(doer,args)
	if doer and doer.Physics then
		doer.Physics:SetMass(args.oldmass)
		doer.Physics:SetCollides(true)
		--clear mask
		--apply the original values
	end
end]]

--Summon spawns a prefab at the target's position
POTIONBASED.Summon = true
POTIONEFFECTS.Summon = function(doer,args,src)
	if args.delayed then
		return true
	end
	local pt
	if src then
		pt = src:GetPosition()
	elseif doer and doer:IsValid() then
		pt = doer:GetPosition()
	end
	
	if pt then
		local guy = SpawnPrefab(args.prefab)
		if guy then
			guy.Transform:SetPosition(pt:Get())
			return true
		end
	end
end
POTIONWEAROFF.Summon = function(doer,args,src)
	--if not args.delayed then
		--return true
	--end
	local pt
	if src then
		pt = src:GetPosition()
	elseif doer and doer:IsValid() then
		pt = doer:GetPosition()
	end
	
	if pt then
		local guy = SpawnPrefab(args.prefab)
		if guy then
			guy.Transform:SetPosition(pt:Get())
			return true
		end
	end
end

--Attract pulls every character and item towards the target (check magnechest?)

--Propulse shoves every character and item away from the target

--Thorns deals 1/4 of taken damage to attacker, but also 1/4 of dealt damage to affected
local function hitme(inst, data)
	if inst and data.target and not inst.alreadythorned then
		inst.alreadythorned = inst:DoTaskInTime(.1,function() inst.alreadythorned = nil end)
		inst.components.combat:GetAttacked(inst,data.damage/4)
	end
end
local function hityou(inst, data)
	if inst and data.attacker and not data.attacker.alreadythorned then
		data.attacker.alreadythorned = data.attacker:DoTaskInTime(.1,function(guy) guy.alreadythorned = nil end)
		inst.alreadythorned = inst:DoTaskInTime(.1,function() inst.alreadythorned = nil end)
		data.attacker.components.combat:GetAttacked(inst,data.damage/4)
	end
end
POTIONEFFECTS.Thorns = function(doer,args)
	if doer and doer.components.combat then
		SpawnPrefab("thornsfx").Transform:SetPosition(doer:GetPosition():Get())
		doer:ListenForEvent("attacked",hityou)
		doer:ListenForEvent("onhitother",hitme)
		return true
	end
end
POTIONWEAROFF.Thorns = function(doer,args)
	if doer and doer.components.combat then
		SpawnPrefab("thornsfx").Transform:SetPosition(doer:GetPosition():Get())
		doer:RemoveEventCallback("attacked",hityou)
		doer:RemoveEventCallback("onhitother",hitme)
	end
end

--Soul gives a chance for a soul to drop
local function onsacrificed(inst, data)
	if inst and math.random() < (inst.souldrop_args.chance or .3) then
		local soul
		if inst.souldrop_args.soul then
			soul = SpawnPrefab(inst.souldrop_args.soul)
		elseif inst:HasTag("epic") then
			soul = SpawnPrefab("soulitem_giant")
		else
			soul = SpawnPrefab("soulitem")
		end
		
		if soul then
			soul.Transform:SetPosition(inst.Transform:GetWorldPosition())
			--[[if soul.components.inventoryitem then --bounce
				soul.components.inventoryitem:OnDropped(true)
			end]]
		end
	end
end
POTIONEFFECTS.Soulbite = function(doer,args)
	if doer and doer.components.health then
		if IsSoulless(doer) then
			return false --fail
		end
		if not args.nofx then
			SpawnPrefab("soulbite").Transform:SetPosition(doer:GetPosition():Get())
		end
		doer.souldrop_args = args
		doer:ListenForEvent("death",onsacrificed)
		return true
	end
end
POTIONWEAROFF.Soulbite = function(doer,args)
	if doer and doer.components.health then
		if not args.nofx then
			local fx = SpawnPrefab("sanity_lower")
			fx.Transform:SetPosition(doer.Transform:GetWorldPosition())
			fx.AnimState:SetAddColour(1,1,1,.6)
		end
		doer.souldrop_args = nil
		doer:RemoveEventCallback("death",onsacrificed)
	end
end