--custom behaviour, makes the instance move around its combat target and shoot periodically

CircleAndShoot = Class(BehaviourNode, function(self, inst, max_dist, min_dist, cooldown, max_attacks)
    BehaviourNode._ctor(self, "CircleAndShoot")
    self.inst = inst
	
    self.max_distsq = max_dist * max_dist
    self.min_distsq = min_dist * min_dist
	
    self.cooldown = cooldown or 2
	self.oncooldown = false
	
    self.max_attacks = max_attacks
    self.numattacks = 0
end)

function CircleAndShoot:__tostring()
    return string.format("target %s", tostring(self.inst.components.combat.target))
end


function CircleAndShoot:Visit()
	
    local combat = self.inst.components.combat
	
    if self.status == READY then

        if combat.target and combat.target:IsValid() then
			self.inst.Physics:Stop()
            self.numattacks = 0
            self.status = RUNNING
            self.inst:AddTag("CircleAndShoot")
        else
            self.inst:RemoveTag("CircleAndShoot")
            self.status = FAILED
        end
    end

    if self.status == RUNNING then
        
        if not combat.target or not combat.target:IsValid() then
            self.status = FAILED
            combat:SetTarget(nil)
            self.inst.components.locomotor:Stop()
            self.inst:RemoveTag("CircleAndShoot")
            return
			
        elseif combat.target.components.health and combat.target.components.health:IsDead() then
            self.status = SUCCESS
            combat:SetTarget(nil)
            self.inst.components.locomotor:Stop()
            self.inst:RemoveTag("CircleAndShoot")
            return
			
        else
            local hp = combat.target:GetPosition()
            local pt = self.inst:GetPosition()
            local dsq = distsq(hp, pt) --Distance to target
			
			if dsq < self.min_distsq then --Too close!
				self.status = FAILED
				--combat:SetTarget(nil)
				self.inst.components.locomotor:Stop()
				self.inst:RemoveTag("CircleAndShoot")
				if self.inst.OnCornered then
					self.inst:OnCornered()
				end
				return
				
			end
			
			
			--This is the actual process.
			
            if self.inst.sg and self.inst.sg:HasStateTag("canrotate") then
				self.inst:FacePoint(hp)
            end
			
			if not self.inst.sg:HasStateTag("attack") then
				
				if self.max_attacks and self.numattacks >= self.max_attacks then
					self.status = SUCCESS
					self.inst.components.locomotor:Stop()
					--self.inst.Physics:Stop()
					self.inst:RemoveTag("CircleAndShoot")
					return
					
				elseif not (dsq > self.max_distsq) and not self.oncooldown then
					
					self.inst:PushEvent("shoot")
					self.numattacks = self.numattacks + 1
					
					self.oncooldown = true
					self.inst:DoTaskInTime(self.cooldown,function()
						if self and self.oncooldown then
							self.oncooldown = false
						end
					end)
					
				else
					self.inst:PushEvent("movetoside")
				end
			end

            self:Sleep(.125)
            
        end
        
    end
end
