require "behaviours/standstill"
require "behaviours/faceentity"
require "behaviours/doaction"
require "behaviours/follow"
require "behaviours/chaseandram"
require "behaviours/circleandshoot" --custom behaviour

local START_FACE_DIST = 15

local MAX_DIST = 20
local TARGET_DIST = 8

local MAX_RAM_TIME = 3
local GIVE_UP_RAM_DIST = 10
local MAX_RAM_DIST = 16
local MAX_RAMS = 5

local CORNERED_DIST = 2
local COOLDOWN = 1
local MAX_SHOTS = 11

local NO_TAGS = {"FX", "NOCLICK", "DECOR", "INLIMBO"}

local function GoHome(inst)
    if inst.shouldGoAway then
        return BufferedAction(inst, nil, ACTIONS.GOHOME)
    end
end

local function GetFaceTargetFn(inst)
    if inst.sg:HasStateTag("busy") then return end

    local target = GetClosestInstWithTag("player", inst, START_FACE_DIST)
    if target and not target:HasTag("notarget") then
        return target
    end
end

local function KeepFaceTargetFn(inst, target)
    if inst.sg:HasStateTag("busy") then return false end

    return not target:HasTag("notarget")
end

local function GetTarget(inst)
	return inst.components.combat.target
end

local function GetDist(inst)
	local target = GetTarget(inst)
	if not target then return false end
	return distsq(inst:GetPosition(),target:GetPosition())
end

--

local DirectorNode = Class(BehaviourNode,function(self, name, children, idxfn)
    BehaviourNode._ctor(self, name or "Director", children)
	assert( idxfn ~= nil, "DirectorNode requires a directing function")
    self.idxfn = idxfn
    self.idx = 1
end)

function DirectorNode:DBString()
	local res = self.idxfn() or -1
    return string.format("index %d idxfn %d",self.idx, res)
end

function DirectorNode:Reset()
    self._base.Reset(self)
    self.idx = 1
end

function DirectorNode:Visit()
	
    if self.status ~= RUNNING then
        self.idx = self.idxfn() or 1
    end
    
    local child = self.children[self.idx]
	if child then
		
		self.status = RUNNING
		
		if child.status == FAILED or child.status == SUCCESS then
			--print("RESET",self.idx)
			child:Reset()
		end
		child:Visit()
		
		if child.status == SUCCESS then
			--print("SUCCESSFUL",self.idx)
            self.status = SUCCESS
        elseif child.status == FAILED then
			self.idx = self.idxfn() or 1
        end
	else
		--print("INVALID CHILD",self.idx)
		self.idx = self.idxfn() or 1
	end
end

--

local ElderBrain = Class(Brain, function(self, inst)
    Brain._ctor(self, inst)
end)

function ElderBrain:OnStart()

    local root =
        PriorityNode(
        {
			--prevent crashes from removing locomotor
            ConditionNode(function() return self.inst.components.health:IsDead() end, "Dead"),
			
            WhileNode(function() return self.inst.shouldGoAway end, "GoAway",
                DoAction(self.inst, GoHome)
			),
			
			Follow(self.inst, GetTarget, 0, TARGET_DIST, MAX_DIST, false),
			
			WhileNode(function() return GetTarget(self.inst) ~= nil end, "ShouldAttack",
				DirectorNode("AttackSwitcher(ElderEye)",{
					ChaseAndRam(self.inst, MAX_RAM_TIME, GIVE_UP_RAM_DIST, MAX_RAM_DIST),
					CircleAndShoot(self.inst, MAX_DIST, CORNERED_DIST, COOLDOWN, MAX_SHOTS),
				},
				function()
					return self.inst.shootmode and 2 or 1
				end
				)
			),
			
            FaceEntity(self.inst, GetFaceTargetFn, KeepFaceTargetFn),
			StandStill(self.inst),
        },.5)
    
    self.bt = BT(self.inst, root)
         
end

return ElderBrain