--[[

Written by Mobbstar
Why do I even bother writing this here? I don't know.

------------------------------
NOTICE FOR USAGE IN OTHER MODS
------------------------------

Place the following code in modmain.lua to make this file do its magic:


local modmusic = nil
function GLOBAL.GetModMusicManager()
    if not modmusic then
		local player = GLOBAL.GetPlayer()
		if player then
			modmusic = player.components.modmusicmanager
		end
    end
    return modmusic
end

AddSimPostInit(function(inst)
	local player = GLOBAL.GetPlayer()
	if player and not player.components.modmusicmanager then
		player:AddComponent("modmusicmanager")
	end
end)

]]

local function sortfn(a,b)
	return a.priority > b.priority
end
local PERIOD = .5 --TODO set back to .2


local ModMusic = Class(function(self, inst)
    self.inst = inst
	
	assert(inst ~= nil,"FALSE SETUP of ModMusicManager!\nNo instance?!")
	if inst ~= GetPlayer() then
		print("FALSE SETUP of ModMusicManager!\nWrong instance:",inst)
	end
    
    self.enabled = true
    --self.doeval = false --not required
    
	self.tracks = {}
	self.currenttrack = nil
	
	self.updatetask = self.inst:DoPeriodicTask(PERIOD,function() self:OnUpdate(PERIOD) end)
    --self.inst:StartUpdatingComponent(self)
	
	 --Take over Woodie's beaver theme
	if self.inst.components.beaverness then
		local hoedown = self:AddTrack("dontstarve/music/music_hoedown",4)
		--local hoedown = {priority = 4, playing = false, doplay = false}
		--table.insert(self.tracks, hoedown)
		self.inst:ListenForEvent("beaverstart",function()
			self.inst.SoundEmitter:KillSound("beavermusic")
			self:StartTrack(hoedown)
		end)
		self.inst:ListenForEvent("beaverend",function()
			self:StopTrack(hoedown)
		end)
		self.inst:DoTaskInTime(0,function()
			if self.inst.components.beaverness:IsBeaver() then
				self.inst.SoundEmitter:KillSound("beavermusic")
				self:StartTrack(hoedown)
			end
		end)
	end
end)

-- These allow you to control the entire manager. Use them with caution.

function ModMusic:Enable()
    self.enabled = true
	if not self.updatetask then
		self.updatetask = self.inst:DoPeriodicTask(PERIOD,function() self:OnUpdate(PERIOD) end)
	end
    --self.inst:StartUpdatingComponent(self)
end

function ModMusic:Disable()
    self.enabled = false
	if self.updatetask then
		self.updatetask:Cancel()
	end
	--self.inst:StopUpdatingComponent(self)
    --self.doeval = true
	self:OnUpdate(0)
end

-- Use this to initialise your music.

function ModMusic:AddTrack(soundpath, priority, cooldown_init, cooldown_rep)
	if not soundpath or type(soundpath) ~= "string" then
		print("FALSE USE of AddTrack in ModMusicManager!\nInvalid soundpath:",soundpath)
		return
	end
	
	priority = priority or 0
	cooldown_rep = cooldown_rep or cooldown_init
	
	local track = {
		name = soundpath,
		priority = priority,
		waitonce = cooldown_init,
		waitmore = cooldown_rep,
		playing = false,
		doplay = false,
	}
	
	table.insert(self.tracks, track)
	table.sort(self.tracks, sortfn)
	
	return track
end

-- Use this to control your music. (You will need to call these when desired.)

function ModMusic:StartTrack(id)
	local track
	local idtype = type(id)
	if idtype == "number" then
		track = self.tracks[id]
	elseif idtype == "table" then
		track = id
	end
	
	if track then
		track.doplay = true
		track.wait = track.waitonce
		--self.doeval = true
	else
		print("FALSE USE of StartTrack in ModMusicManager!\nInvalid track or index:",id)
	end
end

function ModMusic:StopTrack(id)
	local track
	local idtype = type(id)
	if idtype == "number" then
		track = self.tracks[id]
	elseif idtype == "table" then
		track = id
	end
	
	if track then
		track.doplay = false
		track.wait = nil
		--self.doeval = true
	else
		print("FALSE USE of StopTrack in ModMusicManager!\nInvalid track or index:",id)
	end
end

function ModMusic:ContinueTrack(id)
	local track
	local idtype = type(id)
	if idtype == "number" then
		track = self.tracks[id]
	elseif idtype == "table" then
		track = id
	end
	
	if track then
		if track.playing then
			track.doplay = true
			if track.waitmore then
				track.wait = math.max(track.wait, track.waitmore)
			end
		end
	else
		print("FALSE USE of ContinueTrack in ModMusicManager!\nInvalid track or index:",id)
	end
end

-- Everything below here is internal only and should not be used externally.

--self.inst.SoundEmitter:SetParameter( "activetrack", "intensity", 0 )
function ModMusic:FadeOut(track)
	assert(track ~= nil,"CRITICAL ERROR in FadeOut in ModmusicManager!")
	--print("DEBUG| Fading out...")
	track.playing = false
	track.doplay = false
	track.wait = nil
	if track == self.currenttrack then
		--print("DEBUG| No longer playing",track.name)
		self.inst.SoundEmitter:KillSound("activetrack")
		self.currenttrack = nil
	end
end

function ModMusic:FadeIn(track)
	assert(track ~= nil,"CRITICAL ERROR in FadeIn in ModmusicManager!")
	--print("DEBUG| Fading in...")
	track.playing = true
	track.doplay = true
	if not self.currenttrack or track.priority > self.currenttrack.priority then
		--print("DEBUG| Now playing",track.name)
		if track.name then
			self.inst.SoundEmitter:KillSound("activetrack")
			self.inst.SoundEmitter:PlaySound(track.name,"activetrack")
		end
		self.currenttrack = track
	end
end


function ModMusic:OnUpdate(dt)
	dt = dt or 0
	
	--print("DEBUG| NEW UPDATE ------------ ")
	--self.doeval = false
	
	-- handle dynamic music as by game
	-- notice to other modders: you can force-enable and force-disable dynamic music
	if self.inst.components.dynamicmusic.forceenabled then
		self:Disable()
		return
	elseif self.enabled and self.currenttrack then
		self.inst.components.dynamicmusic:Disable()
	elseif not self.inst.components.dynamicmusic.forcedisabled then
		self.inst.components.dynamicmusic:Enable()
	end
	
	-- we sort this table, so the loop begins with the highest priority
	for k,v in ipairs(self.tracks) do
		--print("DEBUG| Testing Track",v.name)
		--do time limits first
		if v.wait then
			v.wait = v.wait - dt
			if v.wait <= 0 then
				self:FadeOut(v)
				--print("DEBUG| Time out")
			end
		end
		
		if not self.enabled then
			self:FadeOut(v)
			--print("DEBUG| Disabled!")
			
		else--if self.doeval then
			
			if v.playing and not v.doplay then
				self:FadeOut(v)
			elseif v.doplay then --this includes the case of a higher priority stopping
				self:FadeIn(v)
			end
			
		end
	end --end loop
end


return ModMusic
