local nicknack = require("sa_nicknack")

local assets = {
	Asset("ANIM","anim/destiller.zip"),

	Asset("IMAGE","images/potionslot.tex"),
	Asset("ATLAS","images/potionslot.xml"),
}

local widgetbuttoninfo = {
	text = "Brew",
	position = Vector3(0,0,0),
	fn = function(inst)
		inst.components.cooldown:StartCharging(math.random(20,30))
		inst.components.container.canbeopened = false
		inst.components.container:Close()
		inst.AnimState:PlayAnimation("work_pre")
		inst.AnimState:PushAnimation("workloop",true)
	end,
	
	validfn = function(inst)
		local ingr = inst.components.container.slots[1]
		local prod = inst.components.container.slots[2]
		return ingr and ingr.components.potionbrewable
			and prod and (prod.prefab == "alchemypotion" or prod.prefab == "alchemybottle")
	end,
}

local function SlotImg(inst,data)
	if data.doer and data.doer.HUD then
	data.doer.HUD.controls.containers[inst].inv[2].bgimage:SetTexture("images/potionslot.xml","potionslot.tex")
	end
end

local function Brew(inst)
	local ingr = inst.components.container.slots[1]
	local prod = inst.components.container.slots[2]
	
	inst.components.container.canbeopened = true
	
	if not (ingr and prod) then --presumably fired by loading
		inst.AnimState:PushAnimation("idle")
		return
	end
	
	saPrint("BREWING")
	
	inst.AnimState:PlayAnimation("work_end")
	inst.AnimState:PushAnimation("idle")
	
	--make empty vials actual potions
	if prod.prefab == "alchemybottle" then
		inst.components.container:RemoveItem(prod)
		prod:Remove()
		prod = SpawnPrefab("alchemypotion")
		inst.components.container:GiveItem(prod, 2, nil, true, true)
	end
	
	--insert the effects and stuff
	for _, eff in ipairs(ingr.components.potionbrewable.fx) do
		--lol I forgot to consider natural aversion prior to 1.2
		if POTIONAFFINITY[eff.name] then
			for _,v in ipairs(POTIONAFFINITY[eff.name]) do
				eff.anti[v] = true
			end
		end
		
		prod:AddEffect(eff)
		--saPrint("Added Effect Done")
	end
	if ingr.components.potionbrewable.tags then
		prod.alchemytags = prod.alchemytags or {}
		for tag, val in pairs(ingr.components.potionbrewable.tags) do
			if not prod.alchemytags[tag] then
				prod.alchemytags[tag] = 0
			end
			prod.alchemytags[tag] = prod.alchemytags[tag] + val
		end
	end
	if ingr.components.potionbrewable.customfn then
		ingr.components.potionbrewable.customfn(prod,ingr)
	end
	
	ingr:Remove()
	
	prod:NewImage(math.random(10),STRINGS.POTION_DESC[math.random(#STRINGS.POTION_DESC)]) --random appearance
	
	--effect-based name and description
	local name, desc;
	
	if not prod.forcedname then
		for _,eff in ipairs(prod.fx) do
			if not name or eff.strength > name.strength then
				name = eff
			elseif not desc or eff.strength > desc.strength then
				desc = eff
			end
		end
	end

	if name then
		prod.components.named:SetName(STRINGS.POTION_NAMES[name.name])
	end
	if desc and STRINGS.POTION_DESC[desc.name] then
		if type(STRINGS.POTION_DESC[desc.name]) == "table" then
			prod.components.inspectable:SetDescription(STRINGS.POTION_DESC[desc.name][math.random(#STRINGS.POTION_DESC[desc.name])])
		else
			prod.components.inspectable:SetDescription(STRINGS.POTION_DESC[desc.name])
		end
	end
	
	--check whether the potion meets criteria for rare special easter egg items
	if CFG_SA.SoftRec ~= "OFF" then
		for k,rec in ipairs(nicknack) do
			--saPrint("TESTING synthesis",rec.name)
			local passed = true
			--test phase 1
			if rec.tags then
				if prod.alchemytags then
					for tag,num in pairs(rec.tags) do
						--saPrint("SEARCHING tag:",tag,num,prod.alchemytags[tag])
						if not prod.alchemytags[tag] or prod.alchemytags[tag] < num then
							--saPrint("ABORT!")
							passed = false
							break
						end
					end
				else
					passed = false
				end
			end
			if passed then --test phase 2
				--saPrint("Tags are alright")
				if rec.effects then
					local t_fx = {}
					for _,eff in ipairs(prod.fx) do
						t_fx[eff.name] = eff.strength
					end
					for eff,num in pairs(rec.effects) do
						--saPrint("SEARCHING eff:",eff,num,t_fx[eff])
						if not t_fx[eff] or t_fx[eff] < num then
							--saPrint("ABORT!")
							passed = false
							break
						end
					end
				end
				if passed then --test phase 3
					--saPrint("Effects are alright")
					if not rec.conditionfn or rec.conditionfn(prod) then
						saPrint("SYNTHESISED in the distiller:",rec.name)
						 --success, do the thing and quit
						 
						local synth
						if rec.product then
							synth = SpawnPrefab(rec.product)
							if synth then
								prod:Remove()
								inst.components.container:GiveItem(synth,2,nil,nil,true)
							end
						end
						if rec.productfn then
							rec.productfn(synth or prod)
						end
						if rec.feat and GetWorld().components.feattrigger then
							GetWorld().components.feattrigger:Trigger("BadSynth")
						end
						
						return
					end
				end
			end
		end
	end
	
	if prod.fx then
		saPrint("END OF BREWING - CURRENT EFFECTS:")
		for k, eff in ipairs(prod.fx) do
			saPrint(eff.name,eff.strength)
		end
		saPrint("-")
	end
end

local function onhit(inst, worker)
	inst.AnimState:PlayAnimation("hit")
	inst.AnimState:PushAnimation(inst.components.cooldown.charged and "idle" or "workloop")
end

local function onhammered(inst, worker)
	if inst:HasTag("fire") and inst.components.burnable then
		inst.components.burnable:Extinguish()
	end
	inst.components.lootdropper:DropLoot()
	SpawnPrefab("collapse_big").Transform:SetPosition(inst.Transform:GetWorldPosition())
	inst:Remove()
	inst.SoundEmitter:PlaySound("dontstarve/common/destroy_wood")
end

local function onbuilt(inst)
	inst.AnimState:PlayAnimation("place")
	inst.AnimState:PushAnimation("idle")
end

local function onsave(inst, data)
	if inst:HasTag("burnt") or inst:HasTag("fire") then
        data.burnt = true
    end
end

local function onload(inst, data)
	if data and data.burnt then
        inst.components.burnable.onburnt(inst)
    end
end

local function loadpostpass(inst)
	if not inst.components.cooldown.charged then
		inst.AnimState:PushAnimation("workloop",true)
		inst.components.container.canbeopened = false
	end
end

local function fn(inst)
	local inst = CreateEntity()
	inst.entity:AddTransform()
	inst.entity:AddAnimState()
	inst.entity:AddSoundEmitter()
	
	inst.AnimState:SetBank("destiller")
	inst.AnimState:SetBuild("destiller")
	inst.AnimState:PlayAnimation("idle")
	
    MakeObstaclePhysics(inst, .7)
	
	inst:AddComponent("inspectable")
	
	inst:AddComponent("cooldown")
	inst.components.cooldown.charged = true
	inst.components.cooldown.cooldown_duration = 20
	inst.components.cooldown.onchargedfn = Brew
	
	inst:AddComponent("container")
	inst.components.container.numslots = 2
	inst.components.container.acceptsstacks = false
	inst.components.container.type = "cooker"
    inst.components.container.widgetanimbank = "ui_cookpot_1x4"
    inst.components.container.widgetanimbuild = "ui_cookpot_1x4"
	inst.components.container.widgetslotpos = {Vector3(0,70,0),Vector3(0,-70,0)}
	inst.components.container.widgetpos = Vector3(200,0,0)
	inst.components.container.side_align_tip = 100
	inst.components.container.widgetbuttoninfo = widgetbuttoninfo
--	inst.components.container.itemtestfn = itemtest
	inst:ListenForEvent("onopen",SlotImg)
	
	inst:AddComponent("lootdropper")
	inst:AddComponent("workable")
	inst.components.workable:SetWorkAction(ACTIONS.HAMMER)
	inst.components.workable:SetWorkLeft(4)
	inst.components.workable:SetOnFinishCallback(onhammered)
	inst.components.workable:SetOnWorkCallback(onhit)
	
	if IsDLCEnabled(REIGN_OF_GIANTS) then
		MakeLargeBurnable(inst, nil, nil, true)
		MakeLargePropagator(inst)
		
		inst.OnSave = onsave 
		inst.OnLoad = onload
	end
	
	inst.OnLoadPostPass = loadpostpass
	
	inst:AddTag("structure")
	inst:AddTag("alchemy")
	
	inst:ListenForEvent( "onbuilt", onbuilt)
	
	return inst
end

return Prefab( "common/objects/alchemydistiller", fn, assets),
	MakePlacer("common/objects/alchemydistiller_placer","destiller","destiller","idle")