local assets = {
	Asset("ANIM","anim/alchemypotion.zip"),
	
	Asset("IMAGE","images/alchemypotion.tex"),
	Asset("ATLAS","images/alchemypotion.xml"),
}

local function OnSave(inst,data)
	data.fx = inst.fx
	data.anim = inst.anim
	data.str = inst.components.inspectable.description
	data.name = inst.components.named.name
	data.alchemytags = inst.alchemytags
	data.uses = inst.remaininguses
	data.forcedname = inst.forcedname
end

local function OnLoad(inst, data)
	if data then
		if data.fx and #data.fx > 0 then
			inst.fx = data.fx
		else --legacy support I guess
			local vial = SpawnPrefab("alchemybottle")
			vial.Transform:SetPosition(inst.Transform:GetWorldPosition())
			inst:Remove()
			return
		end
		inst.components.named:SetName(data.name)
		inst.forcedname = data.forcedname
		inst.alchemytags = data.alchemytags
		inst.remaininguses = data.uses or 2
		if data.anim then
			--apply this stuff
			inst:NewImage(data.anim,data.str)
		end
	end
end

local function onsip(inst,doer) --REVIEW
	inst.fx = inst.fx or {}
	if PotionManager then
		for _,v in pairs(inst.fx) do
			saPrint("Applying",v.name)
			PotionManager:Track(doer,v,inst) --handles ALL the inits from now on
		end
		
		if doer.components.sanity then
			local penalty = #inst.fx
			if penalty > 3 then --decrease the sanity penalty for greater potions
				penalty = (#inst.fx / 2) + 1.5 --halves the penalty for additional effects
			end
			doer.components.sanity:DoDelta(math.min(penalty,5) * CFG_SA.PotionSanity) --hard cap
		end
	end
end

local function cansip(inst,doer)
	return doer and doer.components.combat and inst.fx and #inst.fx > 0
end

local function fn(inst)
	local inst = CreateEntity()
	inst.entity:AddTransform()
	inst.entity:AddAnimState()
	inst.entity:AddSoundEmitter()
	
	inst.AnimState:SetBank("alchemypotion")
	inst.AnimState:SetBuild("alchemypotion")
	
    MakeInventoryPhysics(inst)
	
	inst:AddComponent("inventoryitem")
	inst.components.inventoryitem.atlasname = "images/alchemypotion.xml"
	inst:AddComponent("inspectable")
	
	inst:AddComponent("workable")
	inst.components.workable.onfinish = function(inst)
		local x,y,z = inst:GetPosition():Get()
		local ents = TheSim:FindEntities(x,y,z,3,{"hascombatcomponent"},{"FX"})
		for _, v in pairs(ents) do
			onsip(inst,v) --skip the sipping animation stuff
		end
		
		inst.SoundEmitter:PlaySound("dontstarve/creatures/egg/egg_cold_freeze") --best sound :P
		local shat = SpawnPrefab("shatter")
		shat.Transform:SetPosition(x,y,z)
		shat.components.shatterfx:SetLevel(1)
		
		inst:Remove()
	end
	inst.components.workable.action = ACTIONS.HAMMER
	inst.components.workable:SetWorkLeft(1)
	
	inst.remaininguses = 2
	
	inst:AddComponent("potionactions") --for sippability
	inst.components.potionactions.onsip = onsip
	inst.components.potionactions.cansip = cansip
	
	inst:AddComponent("named")
	
	inst:AddTag("potion")
	inst:AddTag("alchemy")
	inst:AddTag("magic")
	
	inst.fx = {}
	inst.alchemytags = {}
	
	function inst:NewImage(anim,str)
		if not CFG_SA.PotionVar then --no differences allowed! Standart looks
			anim = "p1"
			str = nil
		elseif CFG_SA.PotionVar == "DESC" then
			anim = "p1"
		elseif CFG_SA.PotionVar == "HUE" then
			str = nil
		end
		if #inst.fx == 0 then --it's an empty potion
			anim = "new"
			str = nil
		end
		if type(anim) == "number" then
			anim = "p".. (anim-1)
		end
		inst.anim = anim
		inst.AnimState:PlayAnimation(anim)
		inst.components.inventoryitem.imagename = anim
		
		inst.components.inspectable:SetDescription(str)
	end
	
	function inst:Solve(eff)
		saPrint("REMOVING effect...",eff.name)
		if eff and eff.alt then
			saPrint("Adding alternate effects")
			for _, v in pairs(eff.alt) do
				inst:AddEffect(v)
			end
		end
	end
	
	function inst:AddEffect(eff)
		if not eff then return end
		--test whether there's a conflict
		local delayed = {}
		inst.fx = inst.fx or {}
		
		saPrint("EFFECT to be added:",eff.name,eff.strength)
		for k, otherfx in ipairs(inst.fx) do
			saPrint("FOUND another effect:",otherfx.name)
			if eff.anti then
				for sol, _ in pairs(eff.anti) do
					if otherfx.name == sol then
						saPrint("CONFLICT between",eff.name,eff.strength,otherfx.name,otherfx.strength)
						--see which ones we can remove
						if otherfx.anti and otherfx.anti[eff.name] then
							if otherfx.strength > eff.strength then
								inst:Solve(eff)
								return false --it was hopeless, the effect was too weak
							elseif otherfx.strength == eff.strength then
								table.remove(inst.fx,k)
								inst:Solve(eff)
								inst:Solve(otherfx)
								return false --we just about couldn't add the effect
							end
						else
							inst:Solve(eff)
							return false --we can't add the effect
						end
					end
				end
			end
			if otherfx.anti and otherfx.anti[eff.name] then
			saPrint("Conflict, queueing removal of old effect",otherfx.name)
				table.insert(delayed,k,otherfx)
			end
			if otherfx.name == eff.name then
			saPrint("Conflict between the same kind of effect")
				--see whether we can remove one
				if otherfx.strength > eff.strength then
					inst:Solve(eff)
					return false --the effect is already present in a stronger form
				elseif otherfx.strength < eff.strength then
					table.insert(delayed,k,otherfx) --remove the negligible one
				end
			end
		end
		
		table.insert(inst.fx,eff)
		saPrint("Added Effect",eff.name)
		
		--table.reverse(delayed)
		local newdel = {}
		for i,v in pairs(delayed) do
			newdel[#inst.fx - i] = v
		end
		for k,otherfx in pairs(newdel) do
			table.remove(inst.fx,k)
			inst:Solve(otherfx)
		end
		return true
	end

	inst:NewImage(math.random(10),STRINGS.POTION_DESC[math.random(#STRINGS.POTION_DESC)]) --EDIT
	
	inst.OnSave = OnSave
	inst.OnLoad = OnLoad
	
	return inst
end

return Prefab("common/inventory/alchemypotion", fn, assets)