local assets = {
	Asset("ANIM", "anim/alchemysolvant.zip"),
	Asset("ATLAS","images/alchemysolvant.xml"),
	Asset("IMAGE","images/alchemysolvant.tex"),
}

local function OnFuelOut(inst)
	inst.AnimState:PlayAnimation("work_end")
	inst:DoTaskInTime(.2,function(inst)
		GetPlayer().components.playercontroller:ShakeCamera(inst,"VERTICAL",.3,.1,.5, 24)
	end)
	inst.AnimState:PushAnimation("idle")
end

local function StartWork(inst)
	inst.AnimState:PlayAnimation("work_pre")
	inst.AnimState:PushAnimation("work",true)
	inst.components.fueled:StartConsuming()
end

local function onhammered(inst, worker)
	if not inst.components.fueled:IsEmpty() then
		inst.components.fueled:StopConsuming()
		--inst.components.fueled:MakeEmpty()
		OnFuelOut(inst)
		if worker then
			local hammer = worker.components.inventory
				and (worker.components.inventory:GetActiveItem()
				or worker.components.inventory.equipslots[EQUIPSLOTS.HANDS])
			if hammer and hammer.components.tool then
				inst:Solve(hammer)
			elseif worker.components.health then
				worker.components.health:DoDelta(-worker.components.health.maxhealth / 4)
			end
		end
	end
	--ChangeToInventoryPhysics(inst)
	inst.components.inventoryitem.canbepickedup = true
	inst.components.container.canbeopened = false
    inst.components.fueled.accepting = false
end

local function ondeploy(inst, pt, deployer)
	if pt then
		inst.Physics:Teleport(pt:Get())
	end
	--ChangeToObstaclePhysics(inst)
    inst.SoundEmitter:PlaySound("dontstarve/common/place_structure_stone") --EDIT
	
	inst.components.inventoryitem.canbepickedup = false
	inst.components.container.canbeopened = true
    inst.components.fueled.accepting = true
	inst.components.workable:SetWorkLeft(1)
	if not inst.components.fueled:IsEmpty() then
		StartWork(inst)
	end
end

local function OnLoad(inst,data)
	if data and data.placed then
		inst.components.deployable:Deploy(inst:GetPosition())
	end
end

local function OnSave(inst,data)
	if inst.components.container.canbeopened then --hack?
		data.placed = true
	end
end

local ForcedProducts = {}
local function FindProduct(item)
	--important: more explicit ones at the top, generic ones after those
	if not item then return "spoiled_food" end
	if ForcedProducts[item.prefab] then
		return ForcedProducts[item.prefab]
	elseif item.components.explosive then
		return "bottledignis"
	elseif item:HasTag("wet") then
		return "bottledaqua"
	elseif item.components.perishable and not item.components.edible then
		return "bottledaer"
	elseif item.components.fertilizer or item.components.plantable then
		return "bottledterra"
	elseif item.components.heater or item.components.healer then
		return "bottledignis"
	elseif item.components.insulator or item.components.fishingrod then
		return "bottledaqua"
	elseif item.components.catcher or item.components.hatchable
	or item.components.instrument then
		return "bottledaer"
	elseif item.components.edible and item.components.edible.foodtype == "ELEMENTAL" then
		return "bottledterra"
	elseif item.components.repairer
	or (item.components.weapon and not item.components.weapon.projectile)
	or item.components.edible and item.components.edible.foodtype == "MEAT" then
		return "bottledignis"
	elseif item.components.weapon
	or item.components.edible and item.components.edible.foodtype == "VEGGIE" then
		return "bottledterra"
	elseif item.components.burnable then
		if item.components.fuel and item.components.fuel.fuelvalue <= TUNING.SMALL_FUEL then
			return "bottledaer"
		else
			return "bottledignis"
		end
	end
	
	return "spoiled_food"
end

local function fn()
	local inst = CreateEntity()
	inst.entity:AddTransform()
	inst.entity:AddAnimState()
	inst.entity:AddSoundEmitter()
	
	MakeInventoryPhysics(inst)
	
	inst.AnimState:SetBank("alchemysolvant")
	inst.AnimState:SetBuild("alchemysolvant")
	inst.AnimState:PlayAnimation("idle")
	
	inst:AddComponent("workable")
	inst.components.workable:SetWorkAction(ACTIONS.HAMMER)
	inst.components.workable:SetWorkLeft(0) --hack
	inst.components.workable:SetOnFinishCallback(onhammered)
	
	inst:AddComponent("inspectable")
	inst.components.inspectable.getstatus = function(inst,guy)
		if not inst.components.fueled:IsEmpty()
		and inst.components.container.canbeopened then
			return "ACTIVE"
		end
	end
	
	inst:AddComponent("inventoryitem")
	inst.components.inventoryitem.atlasname = "images/alchemysolvant.xml"
	--inst.components.inventoryitem.imagename = "spellcandle"
	inst.components.inventoryitem.nobounce = true
	
	inst:AddComponent("fueled") --same fiasco as coal generator in IRB, but worse
	inst.components.fueled.maxfuel = TUNING.NIGHTLIGHT_FUEL_START
    inst.components.fueled.accepting = false
    inst.components.fueled.fueltype = "NIGHTMARE"
	inst.components.fueled:SetDepletedFn(OnFuelOut)
	inst.components.fueled.ontakefuelfn = StartWork
	inst.components.fueled:StopConsuming()
	
	inst:AddComponent("deployable")
	inst.components.deployable.ondeploy = ondeploy
	
	--interact with ingredients
	inst:AddComponent("container")
	inst.components.container:SetNumSlots(1)
	inst.components.container.canbeopened = false
	inst.components.container.acceptsstacks = false
	inst.components.container.type = "temp"
	inst.components.container.widgetslotpos = {Vector3(0,0,0)}
	inst.components.container.widgetpos = Vector3(0,600,0)
	inst.components.container.side_align_tip = 100
	inst.components.container.onopenfn = function(inst)
		inst.components.container:Close()
		inst.components.container:DropEverything()
	end
	inst:ListenForEvent("itemget",function(inst,data)
		local item = data.item or inst.components.container.slots[1]
		
		if item and not (item:HasTag("irreplacable") or item:HasTag("rawelement")
		or item:HasTag("soul") or item:HasTag("shadow")
		or inst.components.fueled:TakeFuelItem(item)
		or inst.components.fueled:IsEmpty()) then
			inst:Solve(item)
		end
		
		inst.components.container:DropEverything()
	end)
	
	if GetWorld().components.feattrigger then
		GetWorld().components.feattrigger:Trigger("AlchemySolvant")
	end
	
	function inst.Solve(inst,item)
		if item.fx and #item.fx > 0 then
			local t = {}
			for k,v in pairs(item.fx) do
				table.insert(t,v)
				item.fx[k] = nil --trust me on this one
			end
			for k,v in pairs(t) do
				item:Solve(v)
			end
			item.Transform:SetPosition(inst.Transform:GetWorldPosition())
			if item.components.inventoryitem then
				item.components.inventoryitem:OnDropped(true) --bounce
			end
		else
			local prod = FindProduct(item)
			if prod then
				prod = SpawnPrefab(prod)
				if prod then
					prod.Transform:SetPosition(inst.Transform:GetWorldPosition())
					if prod.components.inventoryitem then
						prod.components.inventoryitem:OnDropped(true) --bounce
					end
				end
			end
			item:Remove()
		end
	end
	
	inst.OnLoad = OnLoad
	inst.OnSave = OnSave
	
	inst:AddTag("alchemy")
	inst:AddTag("magic")
	inst:AddTag("alkahest")
	
	return inst
end

return Prefab("common/inventory/alchemysolvant",fn,assets),
	MakePlacer("common/alchemysolvant_placer","alchemysolvant","alchemysolvant","idle")