--mostly from starpotion.lua

local assets = {
	Asset("ANIM","anim/alchemypotion.zip"),
	
	Asset("IMAGE","images/alchemypotion.tex"),
	Asset("ATLAS","images/alchemypotion.xml"),
}

local prefabs =
{
    "shatter"
}

local function Shatter(inst)
	local pt = inst:GetPosition()
	
	if inst.components.potionactions then
		for k,v in pairs(TheSim:FindEntities(x,y,z,3,nil,{"FX","INLIMBO"})) do
			inst.components.potionactions.onsip(inst,v)
		end
	end
	
	inst.SoundEmitter:PlaySound("dontstarve/creatures/egg/egg_cold_freeze")
	local shat = SpawnPrefab("shatter")
	shat.Transform:SetPosition(pt.x,pt.y,pt.z)
	shat.components.shatterfx:SetLevel(1)
	
	inst:Remove()
end

local function OnSave(inst,data)
	data.uses = inst.remaininguses
end

local function OnLoad(inst, data)
	inst.remaininguses = data.uses or inst.remaininguses
end

local function onsipAqua(inst,doer)
	local guid = tostring(doer.GUID)
	if PotionManager and PotionManager.affected[guid] then
		for k, fx in pairs(PotionManager.affected[guid]) do
			PotionManager:WearOff(guid,k)
		end
		PotionManager.affected[guid] = nil
	end
	inst.remaininguses = inst.remaininguses - 1
	if inst.remaininguses < 1 then
		inst:Remove()
	end
end

local function IncreaseSips(brew)
	if brew and brew.remaininguses and type(brew.remaininguses) == "number" then
		brew.remaininguses = brew.remaininguses + 1
	end
end

local function DecreaseStrengths(brew)
	if brew and brew.fx and type(brew.fx) == "table" then
		for k,v in pairs(brew.fx) do
			if v.strength and type(v.strength) == "number" then
				v.strength = math.max(0,v.strength - 1) --no less than 0
			end
		end
	end
end

local function RemoveRandomEffect(brew)
	if brew and brew.fx and type(brew.fx) == "table" and #brew.fx > 0 then
		local target = math.random(#brew.fx)
		table.remove(brew.fx,target)
	end
end

local function RemoveRandomAversion(brew)
	if brew and brew.fx and type(brew.fx) == "table" then
		for k,v in pairs(brew.fx) do
			if v.anti and type(v.anti) == "table" then
				table.remove(v.anti,math.random(#v.anti))
				return true
			end
		end
	end
end

local function fn()
	local inst = CreateEntity()
	inst.entity:AddTransform()
	inst.entity:AddAnimState()
	inst.entity:AddSoundEmitter()
	
	inst.AnimState:SetBank("alchemypotion")
	inst.AnimState:SetBuild("alchemypotion")
	
    MakeInventoryPhysics(inst)
	
	inst:AddComponent("inspectable")
	inst:AddComponent("inventoryitem")
	inst.components.inventoryitem.atlasname = "images/alchemypotion.xml"
	
	inst:AddComponent("potionbrewable")
	
	inst:AddTag("alchemy")
	inst:AddTag("rawelement")
	inst:AddTag("magic")
	
	return inst
end

local function aquafn()
	local inst = fn()
	
	inst.AnimState:PlayAnimation("p5")
	inst.components.inventoryitem.imagename = "p5"
	
	inst.remaininguses = 3
	
	inst:AddComponent("potionactions")
	inst.components.potionactions.onsip = onsipAqua
	
	inst:AddComponent("workable")
	inst.components.workable.onfinish = Shatter
	inst.components.workable.action = ACTIONS.HAMMER
	inst.components.workable:SetWorkLeft(1)
	
	inst.components.potionbrewable.customfn = RemoveRandomEffect
	
	inst.OnSave = OnSave
	inst.OnLoad = OnLoad
	
	return inst
end

local function ignisfn()
	local inst = fn()
	
	inst.AnimState:PlayAnimation("p6")
	inst.components.inventoryitem.imagename = "p6"
	
	inst.components.potionbrewable.customfn = DecreaseStrengths
	
	return inst
end

local function aerfn()
	local inst = fn()
	
	inst.AnimState:PlayAnimation("p7")
	inst.components.inventoryitem.imagename = "p7"
	
	inst.components.potionbrewable.customfn = IncreaseSips
	
	return inst
end

local function terrafn()
	local inst = fn()
	
	inst.AnimState:PlayAnimation("p4")
	inst.components.inventoryitem.imagename = "p4"
	
	inst.components.potionbrewable.customfn = RemoveRandomAversion
	
	return inst
end

return Prefab("common/inventory/bottledaqua",aquafn,assets,prefabs),
	Prefab("common/inventory/bottledignis",ignisfn,assets,prefabs),
	Prefab("common/inventory/bottledaer",aerfn,assets,prefabs),
	Prefab("common/inventory/bottledterra",terrafn,assets,prefabs)