local assets = {
	Asset("ANIM", "anim/dreamcatcher.zip"),
}

SetSharedLootTable("dreamcatcher", {
	{"dreamcatcher_item", 1},
})

prefabs = {
	"soulitem",
	"soulitem_giant"
}

local function onhammered(inst, worker)
	inst.components.lootdropper:DropLoot()
	SpawnPrefab("collapse_small").Transform:SetPosition(inst.Transform:GetWorldPosition())
	inst.SoundEmitter:PlaySound("dontstarve/common/destroy_wood")
	inst:Remove()
end

local function onhit(inst, worker)
	inst.AnimState:PlayAnimation("hit")
	inst.AnimState:PushAnimation("idle")
end

local function onbuilt(inst)
	inst.AnimState:PlayAnimation("place")
	inst.AnimState:PushAnimation("idle")
end

local function SeekDead(inst)
	local loot
	local pos = inst:GetPosition()
	
	for guid, ent in pairs(TheSim:FindEntities(pos.x,pos.y,pos.z,14)) do --TUNE effect range
		if ent.components.health and ent.components.health:IsDead()
		and not IsSoulless(ent) and not ent:HasTag("dreamcaught") then
			ent:PushEvent("dreamcaught")
			--is it sentient?
			if ent:HasTag("epic") then
				--spawn mighty soul
				loot = SpawnPrefab("soulitem_giant")
				loot.Transform:SetPosition(pos.x,pos.y,pos.z) --get caught in the net
				loot.components.inventoryitem:OnDropped(true) --bounce
				--add tag to avoid super-stacked effects
				ent:AddTag("dreamcaught")
			elseif IsSentient(ent) then
				--spawn soul
				loot = SpawnPrefab("soulitem")
				loot.Transform:SetPosition(pos.x,pos.y,pos.z) --get caught in the net
				loot.components.inventoryitem:OnDropped(true) --bounce
				--add tag to avoid super-stacked effects
				ent:AddTag("dreamcaught")
			end
		end
	end
end


local function fn(Sim)
	local inst = CreateEntity()
	inst.entity:AddTransform()
	inst.entity:AddAnimState()
	inst.entity:AddSoundEmitter()
	
	--local minimap = inst.entity:AddMiniMapEntity()
	--minimap:SetPriority( 1 )
	--minimap:SetIcon( "dreamcatcher.png" )
	
	MakeObstaclePhysics(inst, .35)
	
	inst.AnimState:SetBank("dreamcatcher")
	inst.AnimState:SetBuild("dreamcatcher")
	inst.AnimState:PlayAnimation("idle")
	
	inst:AddComponent("lootdropper")
	inst.components.lootdropper:SetChanceLootTable("dreamcatcher")
	inst:AddComponent("workable")
	inst.components.workable:SetWorkAction(ACTIONS.HAMMER)
	inst.components.workable:SetWorkLeft(3)
	inst.components.workable:SetOnFinishCallback(onhammered)
	inst.components.workable:SetOnWorkCallback(onhit)
	
	inst:AddComponent("inspectable")
	
	inst:ListenForEvent( "onbuilt", onbuilt)
	
	inst:DoPeriodicTask(1.5,SeekDead)
	
	if IsDLCEnabled(REIGN_OF_GIANTS) then
		MakeMediumBurnable(inst)
		MakeLargePropagator(inst)
	end
	
	inst:AddTag("structure")

	return inst
end

local function ondeploy(inst, pt, deployer)
    local ent = SpawnPrefab("dreamcatcher") 
    if ent then 
        pt = Vector3(pt.x, 0, pt.z)
        ent.Physics:SetCollides(false)
        ent.Physics:Teleport(pt.x, pt.y, pt.z) 
        ent.Physics:SetCollides(true)
        ent.SoundEmitter:PlaySound("dontstarve/common/place_structure_wood")
		ent:PushEvent("onbuilt")
        inst:Remove()
    end
end

local function itemfn(Sim)
	local inst = CreateEntity()
	
	inst.entity:AddTransform()
	inst.entity:AddAnimState()
	inst.entity:AddSoundEmitter()
	MakeInventoryPhysics(inst)
	
	inst.AnimState:SetBank("dreamcatcher")
	inst.AnimState:SetBuild("dreamcatcher")
	inst.AnimState:PlayAnimation("item")
	
	inst:AddComponent("inspectable")
	inst.components.inspectable.nameoverride = "EYETURRET_ITEM"
	
	inst:AddComponent("inventoryitem")
	inst.components.inventoryitem.atlasname = "images/soul/dreamcatcher_item.xml"
	
	inst:AddComponent("deployable")
	inst.components.deployable.ondeploy = ondeploy
	inst.components.deployable.placer = "dreamcatcher_placer"
	
	return inst
end

return Prefab("common/objects/dreamcatcher", fn, assets, prefabs),
	Prefab("common/objects/dreamcatcher_item", itemfn, assets),
	MakePlacer("common/dreamcatcher_placer", "dreamcatcher", "dreamcatcher", "idle" )
