require("brains/eldereyebrain")
require("stategraphs/SGeldereye")

------------------- Data -------------------

local assets =
{
	Asset("ANIM", "anim/eldereye.zip"),
	--Asset("SOUND", "sound/moose.fsb"),
}

local prefabs = {
	"spoiled_food",
	"monstermeat",
	"soulitem",
	"hidingblindfold",
}

local loot = { --chance table, this is arbitrary junk really
	spoiled_food = 5,
	monstermeat = 3,
	soulitem = 1,
}

local goodloot = {
	"hidingblindfold",
}

------------------- Music -------------------

local bossthemetrack
local function GetTrack()
	if not bossthemetrack then
		for k,v in pairs(GetModMusicManager().tracks) do
			if v.name == "soulfulalchemy/music/music_boss1" then
				bossthemetrack = v
				break
			end
		end
	end
	return bossthemetrack
end

------------------- AI -------------------

local function RetargetFn(inst)
	if inst.sg:HasStateTag("busy") then return end

	local target
	
	--I don't even know why this thing supports multiple players
	for k,v in pairs(Ents) do
		if v and v.IsValid and v:IsValid() and v:HasTag("player") then
			if not target then
				target = v
			elseif v:GetHorzDistanceSqToInst(inst) < target:GetHorzDistanceSqToInst(inst) then
				target = v
			end
		end
	end

	return target
end

local function KeepTargetFn(inst, target)
	return target:HasTag("player") and inst.components.combat:CanTarget(target)
end

------------------- Callbacks -------------------

local function OnEntitySleep(inst)
	if inst.shouldGoAway then
		inst:Remove()
	elseif inst.components.combat.target then
		--Ripped right from RoG giants...
        local init_pos = inst:GetPosition()
        local player_pos = inst.components.combat.target:GetPosition()
        local angle = inst.components.combat.target:GetAngleToPoint(init_pos)
        local offset = FindWalkableOffset(player_pos, angle*DEGREES, 40, 10)
        local pt = player_pos + offset
        
        if pt and distsq(player_pos, init_pos) > 600 then
            inst:DoTaskInTime(.1, function()
                inst.Transform:SetPosition(pt:Get())
            end)
        end
	end
end

local function OnAttacked(inst, data)
	if data.attacker and data.attacker:HasTag("player") then
		inst.components.combat:SetTarget(data.attacker)
	end --else if too much damage hide briefly?
end

--Reset the target. If one if found, carry on, else, go away.
local function OnTargetDeath(inst, data)
	inst.components.combat:SetTarget()
	inst.components.combat:TryRetarget()
	if inst.components.combat.target == nil then
		--inst:PushEvent("flyaway") --or just set variable and let brain handle the rest?
		inst.shouldGoAway = true
	end
end
local function OnTargetLost(inst, data)
	inst:DoTaskInTime(1,function()
		--inst.components.combat:SetTarget()
		inst.components.combat:TryRetarget()
		if inst.components.combat.target == nil then
			--inst:PushEvent("flyaway") --or just set variable and let brain handle the rest?
			inst.shouldGoAway = true
		end
	end)
end

------------------- Ram attack -------------------

local function OnCollide(inst, other)
    if inst.sg:HasStateTag("ramming") and other and other.components.health
	and not other:HasTag("shadow") then
		inst:DoTaskInTime(2*FRAMES, function() --hurt the other thing
            if other.components.health:GetPercent() >= 0 then
				if not inst.recentlycharged then
					inst.recentlycharged = {}
				end
				
				if inst.recentlycharged[other.GUID] then return end
				
				inst.recentlycharged[other.GUID] = inst:DoTaskInTime(2, function()
					inst.recentlycharged[other.GUID] = nil
				end)
				
				inst.components.combat:DoAttack(other)
            end
		end)
	end
end

local function AdjustSpeed(inst)
	if not inst.components.locomotor then return end
	--the eye should be faster than wilson (this does not affect the projectiles)
	local speed
	local wilson = inst.components.combat.target or GetPlayer()
	if wilson and wilson.components.locomotor then
		speed = wilson.components.locomotor:GetWalkSpeed() + .5
	else
		speed = TUNING.WILSON_RUN_SPEED * (TUNING.CANE_SPEED_MULT * 2)
	end
	inst.components.locomotor.walkspeed = speed
	inst.components.locomotor.runspeed = speed + .5
end

------------------- Save/Load -------------------

local function OnSave(inst, data)
	if not data then return end
	
	data.shouldGoAway = inst.shouldGoAway
	data.lasttime = GetWorld().components.age:GetAge()
	data.shotsdue = inst.shotsdue
	data.shootmode = inst.shootmode
end

local function OnLoad(inst, data)
	if not data then return end
	
	inst.shotsdue = data.shotsdue or 0
	inst.shootmode = data.shootmode
	
	if data.shouldGoAway then
		--inst:PushEvent("flyaway")
		inst.shouldGoAway = true
	elseif data.lasttime then
		inst:DoTaskInTime(0,function() --let it catch up first
			if GetWorld().components.age:GetAge() > data.lasttime + 1 then
				--inst:PushEvent("flyaway")
				inst.shouldGoAway = true
			end
		end)
	end
end

------------------- Main function -------------------

local function fn(Sim)
	local inst = CreateEntity()
	inst.entity:AddTransform()
	inst.entity:AddAnimState()
	inst.entity:AddSoundEmitter()
	inst.entity:AddDynamicShadow()

	inst.Transform:SetScale(2,2,2)

	inst.DynamicShadow:SetSize(2, 1.75)

	MakeCharacterPhysics(inst, 3000, 1)
    inst.Physics:ClearCollisionMask()
    inst.Physics:CollidesWith(COLLISION.WORLD)
    --inst.Physics:CollidesWith(COLLISION.OBSTACLES)
    inst.Physics:CollidesWith(COLLISION.CHARACTERS)

	inst.Physics:SetCollisionCallback(OnCollide)

	inst.Transform:SetFourFaced()
	
	inst.AnimState:SetBank("eldereye")
	inst.AnimState:SetBuild("eldereye")
	inst.AnimState:PlayAnimation("idle_loop", true)
	
	------------------------------------------

	inst:AddTag("soulboss1") --music
	inst:AddTag("epic") --boss soul
	inst:AddTag("monster") --general tag
	inst:AddTag("scarytoprey") --scare rabbits
	inst:AddTag("largecreature") --general tag
	inst:AddTag("flying") --no traps
	inst:AddTag("hostile") --autotarget
	inst:AddTag("shadow") --no "allies" allowed (also no danger sound)
	--note: hitsound breaks due to "largecreature" + "shadow"

	------------------
	
	inst:AddComponent("health")
	inst.components.health:SetMaxHealth(TUNING.DEERCLOPS_HEALTH * .3) --practically 75%, not 30%
	--2.5 times as much partical health due to absorbtion
	inst.components.health.absorb = .6
	inst.components.health.nofadeout = true
	
	------------------

	inst:AddComponent("combat")
	inst.components.combat:SetDefaultDamage(CFG_SA.ELDEREYE_RAMDAMAGE)
	inst.components.combat.playerdamagepercent = CFG_SA.ELDEREYE_PLAYERDAMAGE
	inst.components.combat:SetRange(1.5)
	inst.components.combat.hiteffectsymbol = "eye"
	inst.components.combat:SetAttackPeriod(1)
	inst.components.combat:SetRetargetFunction(1, RetargetFn)
	inst.components.combat:SetKeepTargetFunction(KeepTargetFn)
	--inst.components.combat:SetHurtSound("dontstarve_DLC001/creatures/moose/hurt")
	
	inst:ListenForEvent("killed",OnTargetDeath)
	inst:ListenForEvent("losttarget",OnTargetLost)
	
	------------------------------------------
	
	inst.shouldGoAway = false
	inst.CanDisarm = false
	
	------------------------------------------

	inst:AddComponent("lootdropper")
	inst.loots = loot --custom dropping
	inst.goodloots = goodloot
	inst:AddComponent("inventory")
	
	------------------------------------------

	inst:AddComponent("inspectable")

	------------------------------------------

	--attack variator
	inst.shotsdue = 0
	inst.shootmode = false
	
	function inst:OnShoot()
		inst.shotsdue = inst.shotsdue - 1
		if inst.shotsdue < 1 then
			inst.shootmode = false
		end
	end
	function inst:OnRam()
		inst.shotsdue = inst.shotsdue + 2
		if inst.shotsdue > 9 then
			inst.shootmode = true
		end
	end
	function inst:OnCornered()
		inst.shootmode = false
	end

	------------------------------------------

	inst:ListenForEvent("attacked", OnAttacked)
	inst:ListenForEvent("entitysleep", OnEntitySleep)

	------------------------------------------
	
	MakeLargeBurnableCharacter(inst, "eye", Vector3(110,90,.1))
	MakeHugeFreezableCharacter(inst, "eye", Vector3(110,240,0))

	------------------------------------------

	inst:AddComponent("locomotor")
    --inst.components.locomotor.directdrive = true
	inst:DoPeriodicTask(1,AdjustSpeed)
	AdjustSpeed(inst)
	
	------------------------------------------

	inst:SetStateGraph("SGeldereye")
	local brain = require("brains/eldereyebrain")
	inst:SetBrain(brain)

	inst.OnSave = OnSave
	inst.OnLoad = OnLoad
	
	inst.StopMusic = function(inst)
		if GetModMusicManager() then
			if not FindEntity(inst, 80, nil, {"soulboss1"}) then
				GetModMusicManager():StopTrack(inst.track)
			end
		end
	end
	
	inst:DoTaskInTime(0,function()
		GetModMusicManager():StartTrack(GetTrack())
		inst.track = GetTrack() --for SG
	end)
	inst:DoPeriodicTask(12,function()
		GetModMusicManager():ContinueTrack(GetTrack())
	end)
	
	return inst
end

local function approachfn() --dummy prefab
	local inst = CreateEntity()
	inst.entity:AddTransform()
	
	inst:DoTaskInTime(0,function()
		local theta = math.random() * 2 * PI
		local pt = inst:GetPosition()
		local offset = FindWalkableOffset(pt, theta, 28, 12, true)
		if pt and offset then
			local eye = SpawnPrefab("eldereye")
			if eye then
				eye.Transform:SetPosition((pt + offset):Get())
			end
		end
		inst:Remove()
	end)
	
	inst.persists = false
	
	return inst
end

return Prefab( "common/monsters/eldereye", fn, assets, prefabs),
	Prefab( "common/monsters/eldereyeinit", approachfn)
