local assets = {
	Asset("ANIM", "anim/hungryheart.zip"),
}

prefabs = {
	"soulitem",
	"soulitem_giant"
}

local function TryHarm(inst)
	if inst.components.inventoryitem.owner then return end
	
	local b_anim = true
	local loot
	local pos = inst:GetPosition()
	local hurt = {} --for resetting damage
	
	for _, ent in pairs(TheSim:FindEntities(pos.x,pos.y,pos.z,9)) do
		if not IsSoulless(ent) and not ent:HasTag("hungryheart") then
			ent:PushEvent("hungryheart")
			--is it sentient?
			if ent:HasTag("epic") then
				if b_anim then --play the pump animation
					inst.AnimState:PlayAnimation("idle")
					b_anim = false
				end
				
				if ent.components.health:IsDead() then
					if math.random() < .4 then --chance
						--spawn mighty soul
						loot = SpawnPrefab("soulitem_giant")
						loot.Transform:SetPosition(pos.x,pos.y,pos.z)
						loot.components.inventoryitem:OnDropped(true) --bounce
					end
					
					ent:AddTag("hungryheart") --add tag to avoid super-stacked effects
				else
					if not inst.victims[ent] then
						inst.victims[ent] = 0
					end
					--scaling damage
					inst.victims[ent] = inst.victims[ent] + 15
					ent.components.health:DoDelta(-inst.victims[ent],true,inst.prefab)
					hurt[ent] = true
				end
			elseif IsSentient(ent) then
				if b_anim then --play the pump animation
					inst.AnimState:PlayAnimation("idle")
					b_anim = false
				end
				
				if ent.components.health:IsDead() then
					if math.random() < .4 then --chance
						--spawn soul
						loot = SpawnPrefab("soulitem")
						loot.Transform:SetPosition(pos.x,pos.y,pos.z)
						loot.components.inventoryitem:OnDropped(true) --bounce
					end
					
					ent:AddTag("hungryheart") --add tag to avoid super-stacked effects
				else
					if not inst.victims[ent] then
						inst.victims[ent] = 0
					end
					if ent:HasTag("player") then
						inst.victims[ent] = inst.victims[ent] + 5
						ent.components.health:DoDelta(-inst.victims[ent],true,inst.prefab)
						--warn the player
						SpawnPrefab("soulbite").Transform:SetPosition(ent:GetPosition():Get())
					else
						inst.victims[ent] = inst.victims[ent] + 10
						ent.components.health:DoDelta(-inst.victims[ent],true,inst.prefab)
					end
					hurt[ent] = true
				end
			end
		end
	end
	
	--reset if the thing left
	for ent,damage in pairs(inst.victims) do
		if not hurt[ent] then
			inst.victims[ent] = nil
		end
	end
end

local function onpickup(inst)
	if inst.HarmTask then
		inst.HarmTask:Cancel()
		inst.HarmTask = nil
	end
end

local function ondropped(inst)
	onpickup(inst)
	inst.HarmTask = inst:DoPeriodicTask(4,TryHarm,6)
end

local function fn()
	local inst = CreateEntity()
	inst.entity:AddTransform()
	inst.entity:AddAnimState()
	MakeInventoryPhysics(inst)
	
	inst.AnimState:SetBank("bloodpump01")
	inst.AnimState:SetBuild("hungryheart")
	inst.AnimState:PlayAnimation("idle")
	
	inst:AddComponent("inspectable")
	inst:AddComponent("inventoryitem")
	inst.components.inventoryitem.atlasname = "images/soul/hungryheart.xml"
	
	inst.victims = {}
	
	inst.components.inventoryitem:SetOnDroppedFn(ondropped)
	inst.components.inventoryitem:SetOnPickupFn(onpickup)
	
	ondropped(inst)
	
	return inst
end

return Prefab("common/inventory/hungryheart",fn,assets,prefabs)