local assets = {
	Asset("ANIM","anim/seeds.zip"),
    Asset("ANIM", "anim/koalefant_tracks.zip"),
    Asset("ANIM", "anim/smoke_puff_small.zip"),
	
	Asset("IMAGE","images/mandrake_seeds.tex"),
	Asset("ATLAS","images/mandrake_seeds.xml"),
}

local prefabs =
{
    "small_puff"
}

local function Grow(inst)
	local crop = SpawnPrefab(math.random() < .8 and "mandrake" or "carrot_planted") --TUNE percent chance
	crop.Transform:SetPosition(inst:GetPosition():Get())
	crop:PushEvent("growfromseed",{grower = inst})
	SpawnPrefab("small_puff").Transform:SetPosition(inst:GetPosition():Get())
	inst:Remove()
end

local notags = {'NOBLOCK', 'player', 'FX'}
local function test_ground(inst, pt)
	local tiletype = GetGroundTypeAtPosition(pt)
	local ground_OK = tiletype ~= GROUND.ROCKY and tiletype ~= GROUND.ROAD and tiletype ~= GROUND.IMPASSABLE and
						tiletype ~= GROUND.UNDERROCK and tiletype ~= GROUND.WOODFLOOR and 
						tiletype ~= GROUND.CARPET and tiletype ~= GROUND.CHECKER and tiletype < GROUND.UNDERGROUND
	
	if ground_OK then
	    for k, v in pairs(TheSim:FindEntities(pt.x,pt.y,pt.z, 3, nil, notags)) do
			if v ~= inst and v.entity:IsValid() and v.entity:IsVisible() and not v.components.placer and v.parent == nil then
				if distsq( Vector3(v.Transform:GetWorldPosition()), pt) < 2.25 then
					return false
				end
			end
		end
		return true
	end
	return false
end

local function ondeploy (inst, pt, doer, growtime)
	if pt then
		inst.Transform:SetPosition(pt.x,pt.y,pt.z)
	end
	
	inst.AnimState:SetBank("track")
	inst.AnimState:SetBuild("koalefant_tracks")
	inst.AnimState:PlayAnimation("idle_pile")
	--inst.AnimState:SetRayTestOnBB(false)
	
	inst:RemoveComponent("inventoryitem")
	inst:RemoveComponent("edible")
	inst:RemoveComponent("cookable")
	inst:RemoveComponent("bait")
	inst:RemoveComponent("tradable")
	
	growtime = growtime or TUNING.SAPLING_REGROW_TIME - math.random(TUNING.SAPLING_REGROW_TIME/4)
	inst.growtime = GetTime() + growtime
	inst:DoTaskInTime(growtime,Grow)
end

local function inspect(inst)
    if inst.growtime then
        return "CROP"
    end
end

local function displaynamefn(inst)
    return inst.growtime and STRINGS.NAMES.WORM_DIRT or STRINGS.NAMES.MANDRAKE_SEEDS
end

local function OnSave(inst,data)
	if inst.growtime then
		data.growtime = inst.growtime - GetTime()
	end
end

local function OnLoad(inst,data)
	if data and data.growtime then
		ondeploy(inst,nil,nil,data.growtime)
	end
end

local function fn(inst)
	local inst = CreateEntity()
	inst.entity:AddTransform()
	inst.entity:AddAnimState()
	inst.entity:AddSoundEmitter()
	
	inst.AnimState:SetBank("seeds")
	inst.AnimState:SetBuild("seeds")
	inst.AnimState:PlayAnimation("idle")
	
	inst.AnimState:SetRayTestOnBB(true)
	
    MakeInventoryPhysics(inst)
	
	inst:AddComponent("inventoryitem")
	inst.components.inventoryitem.atlasname = "images/mandrake_seeds.xml"
	
	inst:AddComponent("inspectable")
    inst.components.inspectable.getstatus = inspect
	
	inst:AddComponent("stackable")
	inst.components.stackable.maxsize = TUNING.STACK_SIZE_SMALLITEM
	
	inst:AddComponent("edible")
	inst.components.edible.foodtype = "SEEDS"
	inst.components.edible.healthvalue = TUNING.HEALING_TINY
	inst.components.edible.hungervalue = TUNING.CALORIES_TINY
	
	inst:AddComponent("cookable")
	inst.components.cookable.product = "seeds_cooked"
	
	inst:AddComponent("tradable")
	
	inst:AddComponent("bait")
	
	inst:AddComponent("deployable")
    inst.components.deployable.test = test_ground
    inst.components.deployable.ondeploy = ondeploy
	
    inst.displaynamefn = displaynamefn
	
	inst:AddTag("alchemy")
	inst:AddTag("seeds")
	
	inst.OnSave = OnSave
	inst.OnLoad = OnLoad
	
	return inst
end

return Prefab( "common/inventory/mandrake_seeds", fn, assets),
	MakePlacer("common/inventory/mandrake_seeds_placer", "track", "koalefant_tracks", "idle_pile")