local assets = {
	Asset("ANIM","anim/potiongun.zip"),
	Asset("ANIM","anim/swap_blowdart_pipe.zip"),
	Asset("ANIM","anim/potionspit.zip"),
	
	Asset("IMAGE","images/potiongun.tex"),
	Asset("ATLAS","images/potiongun.xml"),
}

--potion spitter item
-------------------------

local function onattack(inst, attacker, target)
	if not (attacker and attacker.components.inventory) then return end --we can't shoot ourselves!
	
	local v
	local potion
	
	for i = 1,attacker.components.inventory.maxslots do --custom iteration (for left-to-right)
		v = attacker.components.inventory.itemslots[i]
		if v and v.prefab == "alchemypotion" and v.fx and #v.fx > 0 then
			potion = v
			break
		end
	end
	
	--[[attacker.components.inventory:FindItem(function(v) --find all potions
		if v and v.prefab == "alchemypotion" and v.fx and #v.fx > 0 then
			return true
		end
		--return false
	end)]]
	
	if potion then
		--saPrint("PotionGun found a potion")
		--make the projectile (this is a minor madhack)
		local proj = SpawnPrefab("potionspit")
		local x,y,z = attacker.Transform:GetWorldPosition()
		proj.Transform:SetPosition(x,y + 1,z)
		proj.components.projectile:Throw(inst, target, attacker)
		proj.fx = potion.fx
		
		--take an use from the potion
		potion.remaininguses = potion.remaininguses - 1
		if potion.remaininguses < 1 then
			potion:Remove()
		end
	elseif attacker.components.talker then --say that we need a potion
		saPrint("PotionGun out of potions")
		attacker.components.talker:Say(GetString(attacker.prefab, "ANNOUNCE_NEEDPOTION"))
	end
end

local function onequip(inst, owner)
	owner.AnimState:OverrideSymbol("swap_object", "swap_blowdart_pipe", "swap_blowdart_pipe")
	owner.AnimState:Show("ARM_carry") 
	owner.AnimState:Hide("ARM_normal") 
end

local function onunequip(inst, owner) 
	owner.AnimState:ClearOverrideSymbol("swap_object")
	owner.AnimState:Hide("ARM_carry") 
	owner.AnimState:Show("ARM_normal") 
end

local function fn(inst)
	local inst = CreateEntity()
	inst.entity:AddTransform()
	inst.entity:AddAnimState()
	inst.entity:AddSoundEmitter()
	
	inst.AnimState:SetBank("potiongun")
	inst.AnimState:SetBuild("potiongun")
	inst.AnimState:PlayAnimation("idle")
	
    MakeInventoryPhysics(inst)
	
	inst:AddComponent("inventoryitem")
	inst.components.inventoryitem.atlasname = "images/potiongun.xml"
	inst:AddComponent("inspectable")
	
    inst:AddComponent("equippable")
    inst.components.equippable:SetOnEquip(onequip)
    inst.components.equippable:SetOnUnequip(onunequip)
	
	inst:AddComponent("weapon")
    inst.components.weapon:SetDamage(0)
    inst.components.weapon:SetRange(8, 10)
    inst.components.weapon:SetOnProjectileLaunch(onattack)
	inst.components.weapon.projectile = "splash_spiderweb" --or regular splash, idk
	
	inst:AddTag("alchemy")
	--inst:AddTag("gun") --this tag causes an not fully implemented state :-/
	inst:AddTag("blowdart") --this tag fixes the issue above
	inst:AddTag("nopunch")
	
	return inst
end

--potion blob projectile
-------------------------

local function ApplyAll(inst,guy)
	if inst.fx and guy and guy:IsValid() and not guy:HasTag("FX") then
		--apply potion effect
		if PotionManager then
			for _,v in pairs(inst.fx) do
				saPrint("Applying",v.name,"via POTIONGUN on",guy)
				PotionManager:Track(guy,v,inst)
			end
		end
		
		if guy.components.sanity then --unlike sipping potions, this is uncapped
			guy.components.sanity:DoDelta(CFG_SA.PotionSanity * #inst.fx)
		end
	end
end

local function oncollide(inst,thrower,other)
	local x,y,z = inst.Transform:GetWorldPosition()
	
	for _,guy in pairs(TheSim:FindEntities(x,y,z,2)) do
		ApplyAll(inst,guy)
	end
	
	SpawnPrefab("splash").Transform:SetPosition(x,y,z)
	
	inst:Remove()
end

local function blobfn(inst)
	local inst = CreateEntity()
	inst.entity:AddTransform()
	inst.entity:AddAnimState()
	inst.entity:AddSoundEmitter()
	inst.entity:AddDynamicShadow()
	
	inst.AnimState:SetBank("potionspit")
	inst.AnimState:SetBuild("potionspit")
	inst.AnimState:PlayAnimation("idle_loop",true)
	--emit particles (those being tiny blobs)?
	inst.DynamicShadow:SetSize(1.5,1)
	
	MakeInventoryPhysics(inst)
	
    --inst:DoPeriodicTask(.1,oncollide)
	inst:DoTaskInTime(1.5,oncollide)
	
	inst:AddComponent("projectile")
	inst.components.projectile.speed = 10 --TUNE
	--inst:DoTaskInTime(.4,function() inst.components.projectile.speed = 6 end)
	inst.components.projectile.range = 10
	inst.components.projectile.homing = false
	inst.components.projectile.onmiss = oncollide
	inst.components.projectile.onhit = oncollide
	
	inst.persists = false
	
	inst:AddTag("FX")
	inst:AddTag("NOCLICK")
	inst:AddTag("alchemy")
	inst:AddTag("AoE")
	
	return inst
end

return Prefab("common/inventory/potiongun", fn, assets),
	Prefab("common/potionspit", blobfn, assets)