local function fn(inst)
	local inst = CreateEntity()
	inst.entity:AddTransform()
	
	inst.affected = {}
	
	function inst:WearOff(guid,k) --string GUID, index
		local eff = inst.affected[guid][k]
		saPrint("Wearing off",guid,eff and eff.name,k)
		assert(guid and eff and k,"cannot wear off without all data")
		
		POTIONWEAROFF[eff.name](Ents[tonumber(guid)],eff.args)
		table.remove(inst.affected[guid],k)
	end
	
	function inst:Track(guy,eff,brew)
		eff.args = eff.args or {}
		
		--if it is instant, just call the thing done
		if not (eff.dur and POTIONWEAROFF[eff.name]) then
			if POTIONEFFECTS[eff.name] then
				if POTIONBASED[eff.name] and brew then --if it's source-based
					if not eff.firedonsource then --make sure we don't fire this twice
						eff.firedonsource = true
						return POTIONEFFECTS[eff.name](guy,eff.args,brew)
					end
				else
					return POTIONEFFECTS[eff.name](guy,eff.args,brew)
				end
			end
			return false
		end
		
		local guid = tostring(guy.GUID)
		if not inst.affected[guid] then
			inst.affected[guid] = {}
		end
		
		--compare with active effects
		for k, otherfx in pairs(inst.affected[guid]) do
			if otherfx.name == eff.name then
				saPrint("Conflict between the same kind of effect (tracking)")
				--see whether we can remove the other
				if otherfx.strength <= eff.strength then
					inst:WearOff(guid,k)
				else
					return false --prevent applying the same effect several times
				end
			end
		end
		
		saPrint("trying to track effect",eff.name)
		local success = POTIONEFFECTS[eff.name](guy,eff.args,brew)
		
		if success then
			saPrint("TRACKING Effect",eff.name)
			table.insert(inst.affected[guid],{name=eff.name,strength=eff.strength,dur=eff.dur,args=eff.args})
		end
		return success
	end
	
	function inst:OnSave(data) --just like in life...
		local ref = {} --we MAKE referrences but GET data...
		if CFG_SA.DumpEffects then return end
		
		data.activeeffects = {}
		for guid,fx in pairs(inst.affected) do
			local numguid = tonumber(guid)
			data.activeeffects[guid] = {}
			
			for _,eff in pairs(fx) do
				if Ents[numguid] and
				(Ents[numguid].persists or Ents[numguid]:HasTag("player")) then
					saPrint("SAVING EFFECT",eff.name)
					table.insert(data.activeeffects[guid],eff)
				end
			end
		
			--if numguid ~= self.anchor.GUID then
				saPrint("REFERENCING",guid)
				table.insert(ref,numguid)
			--end
		end
		
		if inst.arbitrarydata then
			data.arbitrary = inst.arbitrarydata
		end
		
		if #ref == 0 then ref = nil end
		saPrint("DONE WRITING SAVE DATA, RETURNING REFERENCES")
		
		return ref
	end
	
	function inst:OnLoadPostPass(newents,data)
		saPrint("ATTEMPT LOADING DATA")
		if data then
			inst.arbitrarydata = data.arbitrary
			if not CFG_SA.DumpEffects then
				saPrint("FOUND DATA")
				if not data.activeeffects then --backwards-compatibility
					data.activeeffects = {}
					for guid,fx in pairs(data) do
						if guid ~= "arbitrary" and guid ~= "activeeffects" then
							data.activeeffects[guid] = fx
						end
					end
				end
				
				for guid,fx in pairs(data.activeeffects) do
					local numguid = tonumber(guid)
					if newents[numguid] then
						saPrint("LOADING DATA FOR",newents[numguid].entity)
						local keyguid = tostring(newents[numguid].entity.GUID)
						inst.affected[keyguid] = {}
						for _,v in pairs(fx) do
							table.insert(inst.affected[keyguid],v)
							if POTIONEFFECTS[v.name] then
								POTIONEFFECTS[v.name](newents[numguid].entity,v.args)
							end
						end
					else
						local e = #fx > 0 and "EFFECT #1: ".. fx[1].name or "CAN'T FIND EFFECTS EITHER"
						Print(VERBOSITY.WARNING,"FAILED TO FIND SAVED ENTITY",guid,e)
					end
				end
			end
		end
	end
	
	function inst:OnUpdate(dt)
		--saPrint("UPDATE")
		for guid, fx in pairs(inst.affected) do
			local numguid = tonumber(guid)
			saPrint("PREFAB",Ents[numguid] or guid)
			if not Ents[numguid] then
				saPrint("Couldn't find Entity anymore")
				inst.affected[guid] = nil
			else
				local FeatsCheck --Feats of the world support
				if GetWorld().components.feattrigger
				and Ents[numguid]:HasTag("player") and not Ents[numguid]:HasTag("IGNOREAMBROSIANFEAT") then
					FeatsCheck = {Shield = true,FireImmune = true,IceImmune = true,GrueImmune = true,}
				end --the feat for having all warding effects
				for k,v in pairs(fx) do
					if FeatsCheck then --tick it off the list
						FeatsCheck[v.name] = nil
					end
					v.dur = v.dur - dt
					saPrint(v.name,v.dur)
					if v.dur <= 0 then
						inst:WearOff(guid,k)
					end
				end
				
				if #fx == 0 then
					saPrint("ALL WORE OFF")
					Ents[numguid]:PushEvent("potionworeoff")
					self.affected[guid] = nil
				end
				
				if FeatsCheck then
					for k,v in pairs(FeatsCheck) do
						return --don't do the following if something is left
					end
					GetWorld().components.feattrigger:Trigger("PotionShield")
					Ents[numguid]:AddTag("IGNOREAMBROSIANFEAT") --silly way to reduce lag
				end
			end
		end
	end
	
	inst:DoPeriodicTask(1,function() inst:OnUpdate(1) end) --TUNE tick time (twice)
	
	inst:AddTag("NOCLICK") --idk just to make sure :P
	
	return inst
end

return Prefab("common/potionmanager",fn)