local assets = {
	Asset("ANIM", "anim/researchlab_soul.zip"),
}

SetSharedLootTable("researchlab_soul", {
	{"boards",      1},
	{"boards",      1},
	{"petals_evil", 1},
	{"monstermeat", 1},
	{"soulitem",    0.4},
})

local function onhammered(inst, worker)
	inst.components.lootdropper:DropLoot()
	SpawnPrefab("collapse_small").Transform:SetPosition(inst.Transform:GetWorldPosition())
	inst.SoundEmitter:PlaySound("dontstarve/common/destroy_wood")
	inst:Remove()
end

local function onhit(inst, worker)
	inst.AnimState:PlayAnimation("hit")
	
	if inst.components.prototyper.on then
		inst.AnimState:PushAnimation("proxy", true)
	else
		inst.AnimState:PushAnimation("idle")
	end
end

local function onbuilt(inst)
	inst.components.prototyper.on = true
	inst.AnimState:PlayAnimation("place")
	inst.AnimState:PushAnimation("proxy_pre")
	inst.AnimState:PushAnimation("proxy", true)
end

local function onturnon(inst)
	inst.AnimState:PlayAnimation("proxy_pre")
	inst.AnimState:PushAnimation("proxy", true)
	--inst.SoundEmitter:PlaySound("dontstarve/common/researchmachine_XXX_idle_LP","idlesound")
end

local function onturnoff(inst)
	inst.AnimState:PushAnimation("proxy_end")
	inst.AnimState:PushAnimation("idle")
	inst.SoundEmitter:KillSound("idlesound")
end


local function fn(Sim)
	local inst = CreateEntity()
	inst.entity:AddTransform()
	inst.entity:AddAnimState()
	inst.entity:AddSoundEmitter()
	
	--local minimap = inst.entity:AddMiniMapEntity()
	--minimap:SetPriority( 5 )
	--minimap:SetIcon( name..".png" )
	
	MakeObstaclePhysics(inst, .4)
	
	inst.AnimState:SetBank("researchlab_soul")
	inst.AnimState:SetBuild("researchlab_soul")
	inst.AnimState:PlayAnimation("idle")
	
	inst:AddComponent("lootdropper")
	inst.components.lootdropper:SetChanceLootTable("researchlab_soul")
	inst:AddComponent("workable")
	inst.components.workable:SetWorkAction(ACTIONS.HAMMER)
	inst.components.workable:SetWorkLeft(4)
	inst.components.workable:SetOnFinishCallback(onhammered)
	inst.components.workable:SetOnWorkCallback(onhit)
	
	inst:AddComponent("inspectable")
	
	inst:AddComponent("prototyper")
	inst.components.prototyper.onturnon = onturnon
	inst.components.prototyper.onturnoff = onturnoff
	inst.components.prototyper.trees = TUNING.PROTOTYPER_TREES.SOULBASIC
	
	inst.components.prototyper.onactivate = function()
		inst.AnimState:PlayAnimation("use")
		inst.AnimState:PushAnimation("proxy", true)
		--inst.SoundEmitter:PlaySound("dontstarve/common/researchmachine_"..soundprefix.."_run","sound")

		inst:DoTaskInTime(1.5, function() 
			inst.SoundEmitter:PlaySound("dontstarve/characters/woodie/lucy_transform") --violin rising		
			
			--inst.SoundEmitter:KillSound("sound")
			--inst.SoundEmitter:PlaySound("dontstarve/common/researchmachine_"..soundprefix.."_ding")		
		end)
	end
	
	if IsDLCEnabled(REIGN_OF_GIANTS) then
		MakeMediumBurnable(inst)
		MakeLargePropagator(inst)
	end
	
	inst:ListenForEvent( "onbuilt", onbuilt)
	
	inst:AddTag("prototyper")
	inst:AddTag("structure")
	inst:AddTag("level2")

	return inst
end

local function ondeploy(inst, pt, deployer)
    local ent = SpawnPrefab("researchlab_soul") 
    if ent then 
        pt = Vector3(pt.x, 0, pt.z)
        ent.Physics:SetCollides(false)
        ent.Physics:Teleport(pt.x, pt.y, pt.z) 
        ent.Physics:SetCollides(true)
        ent.SoundEmitter:PlaySound("dontstarve/common/place_structure_wood")
		ent:PushEvent("onbuilt")
        inst:Remove()
    end
end

local function itemfn(Sim)
	local inst = CreateEntity()
	
	inst.entity:AddTransform()
	inst.entity:AddAnimState()
	inst.entity:AddSoundEmitter()
	MakeInventoryPhysics(inst)
	
	inst.AnimState:SetBank("researchlab_soul")
	inst.AnimState:SetBuild("researchlab_soul")
	inst.AnimState:SetPercent("place", 0)
	
	inst:AddComponent("inspectable")
	inst.components.inspectable.nameoverride = "EYETURRET_ITEM"
	
	inst:AddComponent("inventoryitem")
	inst.components.inventoryitem.atlasname = "images/soul/researchlab_soul_item.xml"
	
	inst:AddComponent("deployable")
	inst.components.deployable.ondeploy = ondeploy
	inst.components.deployable.placer = "researchlab_soul_placer"
	
	if GetWorld().components.feattrigger then
		GetWorld().components.feattrigger:Trigger("SoulCraft")
		GetWorld().components.feattrigger:Trigger("LapisLazarus",true)
		GetWorld().components.feattrigger:Trigger("AlchemySolvant",true)
	end
	
	return inst
end

return Prefab("common/objects/researchlab_soul", fn, assets),
	Prefab("common/objects/researchlab_soul_item", itemfn, assets),
	MakePlacer( "common/researchlab_soul_placer", "researchlab_soul", "researchlab_soul", "idle" )
