local assets = {
	Asset("ANIM", "anim/rock_flintless.zip"),
	Asset("ANIM", "anim/cutstone.zip"),
}

SetSharedLootTable("resurrectionlapis", {
	{"soulitem",    1},
	{"soulitem",    1},
	{"soulitem",    0.9},
	{"soulitem",    0.8},
	{"soulitem",    0.7},
	{"rocks",       1},
	{"rocks",       1},
	{"rocks",       1},
	{"rocks",       1},
	{"rocks",       1},
	{"rocks",       1},
})

local function onhammered(inst, worker)
	--if not inst.components.resurrector.used then
		inst.components.lootdropper:DropLoot()
	--end
	SpawnPrefab("collapse_big").Transform:SetPosition(inst.Transform:GetWorldPosition())
	inst.SoundEmitter:PlaySound("dontstarve/common/destroy_stone")
	inst:Remove()

	-- Remove from save index
	SaveGameIndex:DeregisterResurrector(inst)
end

--[[local function onbuilt(inst)
	inst.AnimState:PlayAnimation("place")
	inst.AnimState:PushAnimation("idle", false)
end]]

local function FindAndKill(inst)
	local pos = inst:GetPosition()
	local drained = 0
	
	for guid, ent in pairs(TheSim:FindEntities(pos.x,pos.y,pos.z,14)) do
		if not IsSoulless(ent) and ent.components.health and not ent.components.health:IsDead() then
			--spawn some cool ghost fx!
			local fx = SpawnPrefab("sanity_lower")
			fx.Transform:SetPosition(ent.Transform:GetWorldPosition())
			fx.AnimState:SetAddColour(1,1,1,.6)
			
			drained = drained + ent.components.health.currenthealth
			ent.components.health:Kill()
		end
	end
	
	return drained
end

local function doresurrect(inst, dude)
	inst:AddTag("busy")	
	inst:RemoveComponent("workable")
	--inst.MiniMapEntity:SetEnabled(false)
	local pos = inst:GetPosition()
	
	if inst.Physics then
		MakeInventoryPhysics(inst) -- collides with world, but not character
	end
	
	if dude.components.playercontroller then
		dude.components.playercontroller:Enable(false)
	end
	
	GetClock():MakeNextDay()
	dude.Transform:SetPosition(pos.x,pos.y,pos.z)
	dude:Hide()
	TheCamera:SetDistance(12)
	dude.components.hunger:Pause()
	
	scheduler:ExecuteInTime(3.5, function()
		dude:Show()
		
		--check for creatures to consume
		local bonus = FindAndKill(inst)
		
		if bonus == 0 then
			--if no suitable meat is around, do a really embarrassing effect
			local fx = SpawnPrefab("book_fx")
			fx.Transform:SetPosition(pos.x,pos.y,pos.z)
			
			inst.SoundEmitter:PlaySound("dontstarve/common/shrine/sadwork_explo")
		else
			local fx = SpawnPrefab("explode_small")
			fx.Transform:SetPosition(pos.x,pos.y,pos.z)
			
			inst.SoundEmitter:PlaySound("dontstarve/common/shrine/firework_explo")
		end
		inst.AnimState:PlayAnimation("low")
		
		local base --for calculations
		
		if dude.components.hunger then
			base = 1/3 * dude.components.hunger.max + bonus/5
			dude.components.hunger.current = 0.01 --hack, because we don't use percent
			dude.components.hunger:DoDelta(base, true)
		end
		
		if dude.components.health then
			base = TUNING.RESURRECT_HEALTH/2 + bonus/3
			dude.components.health:Respawn(base)
			dude.components.health:SetInvincible(true)
		end
		
		if dude.components.sanity then
			dude.components.sanity:SetPercent(.5)
		end
		
		dude.components.hunger:Resume()
		
		dude.sg:GoToState("wakeup")
		
		--Feats of the World support
		if GetWorld().components.feattrigger and dude.components.health.currenthealth >= TUNING.WILSON_HEALTH then
			GetWorld().components.feattrigger:Trigger("LapisLazarus")
		end
		
		
		dude:DoTaskInTime(3, function(inst) 
			if dude.HUD then
				dude.HUD:Show()
			end
			
			dude.components.health:SetInvincible(false)
			if dude.components.playercontroller then
				dude.components.playercontroller:Enable(true)
			end
			
			TheCamera:SetDefault()
			inst:RemoveTag("busy")
		end)
		
		inst:DoTaskInTime(7, function()
			local tick_time = TheSim:GetTickTime()
			local time_to_erode = 4
			inst:StartThread( function()
				local ticks = 0
				while ticks * tick_time < time_to_erode do
					local erode_amount = ticks * tick_time / time_to_erode
					inst.AnimState:SetErosionParams( erode_amount, 0.1, 1.0 )
					ticks = ticks + 1
					Yield()
				end
				inst:Remove()
			end)
		end)
	end)
end

local function fn(Sim)
	local inst = CreateEntity()
	inst.entity:AddTransform()
	inst.entity:AddAnimState()
	inst.entity:AddSoundEmitter()
	
	--local minimap = inst.entity:AddMiniMapEntity()
	--minimap:SetPriority( 1 )
	--minimap:SetIcon( "dreamcatcher.png" )
	
	MakeObstaclePhysics(inst, 1)
	
	inst.AnimState:SetBank("rock_flintless")
	inst.AnimState:SetBuild("rock_flintless")
	inst.AnimState:PlayAnimation("full")
	inst.AnimState:SetMultColour(.2,.4,1,1)
	
	inst.Transform:SetScale(1,1.5,1)
	
	inst:AddComponent("resurrector")
	inst.components.resurrector.active = true
	inst.components.resurrector.doresurrect = doresurrect
	--inst.components.resurrector.makeusedfn = makeused
	--inst.components.resurrector.penalty = 1
	
	inst:AddComponent("lootdropper")
	inst.components.lootdropper:SetChanceLootTable("resurrectionlapis")
	inst:AddComponent("workable")
	inst.components.workable:SetWorkAction(ACTIONS.MINE)
	inst.components.workable:SetWorkLeft(5)
	inst.components.workable:SetOnFinishCallback(onhammered)
	
	inst:AddComponent("inspectable")
	
	--inst:ListenForEvent( "onbuilt", onbuilt)
	
	--Feats of the World support
	if GetWorld().components.feattrigger then
		GetWorld().components.feattrigger:Trigger("LapisLazarus",true)
	end
	
	inst:AddTag("structure")
	
	return inst
end

local function ondeploy(inst, pt, deployer)
    local ent = SpawnPrefab("resurrectionlapis") 
    if ent then 
        pt = Vector3(pt.x, 0, pt.z)
        ent.Physics:SetCollides(false)
        ent.Physics:Teleport(pt.x, pt.y, pt.z) 
        ent.Physics:SetCollides(true)
        ent.SoundEmitter:PlaySound("dontstarve/common/place_structure_stone")
		ent:PushEvent("onbuilt")
        inst:Remove()
    end
end

local function itemfn(Sim)
	local inst = CreateEntity()
	
	inst.entity:AddTransform()
	inst.entity:AddAnimState()
	inst.entity:AddSoundEmitter()
	MakeInventoryPhysics(inst)
	
	inst.AnimState:SetBank("cutstone")
	inst.AnimState:SetBuild("cutstone")
	inst.AnimState:PlayAnimation("idle")
	
	inst.AnimState:SetMultColour(.2,.4,1,1)
	
	inst:AddComponent("inspectable")
	inst.components.inspectable.nameoverride = "EYETURRET_ITEM"
	
	inst:AddComponent("inventoryitem")
	inst.components.inventoryitem.atlasname = "images/soul/resurrectionlapis_item.xml"
	
	inst:AddComponent("deployable")
	inst.components.deployable.ondeploy = ondeploy
	inst.components.deployable.placer = "resurrectionlapis_placer"
	
	return inst
end

return Prefab("common/objects/resurrectionlapis", fn, assets),
	Prefab("common/objects/resurrectionlapis_item", itemfn, assets),
	MakePlacer("common/resurrectionlapis_placer", "rock_flintless", "rock_flintless", "full" )
	