local assets = {
	Asset("ANIM", "anim/soulfood.zip"),
	Asset("IMAGE", "images/soul/soulfood.tex"),
	Asset("ATLAS", "images/soul/soulfood.xml"),
}

local prefabs = 
{
	"spoiled_food",
}

local function common()
	local inst = CreateEntity()
	inst.entity:AddTransform()
	inst.entity:AddAnimState()
	
	MakeInventoryPhysics(inst)
	
	inst.AnimState:SetBank("soulfood")
	inst.AnimState:SetBuild("soulfood")
	
	inst:AddComponent("inspectable")
	inst:AddComponent("inventoryitem")
	inst.components.inventoryitem.atlasname = "images/soul/soulfood.xml"
	
	inst:AddComponent("stackable")
	inst.components.stackable.maxsize = TUNING.STACK_SIZE_SMALLITEM
	
	inst:AddComponent("edible")
	
	inst:AddComponent("perishable")
	inst.components.perishable.onperishreplacement = "spoiled_food"
	
	MakeSmallBurnable(inst)
	MakeSmallPropagator(inst)
	
	inst:AddComponent("bait")
	
	inst:AddComponent("tradable")
		
	inst:AddTag("preparedfood")
	inst:AddTag("soul")
	
	return inst
end

local function fn1()
	local inst = common()
	
	inst.AnimState:PlayAnimation("cake")
	
	inst.components.edible.healthvalue = TUNING.HEALING_HUGE
	inst.components.edible.hungervalue = -TUNING.CALORIES_LARGE *1.5
	inst.components.edible.sanityvalue = -TUNING.SANITY_TINY
	inst.components.edible.foodtype = "VEGGIE"
	
	inst.components.perishable:SetPerishTime(TUNING.PERISH_MED)
	inst.components.perishable:StartPerishing()
	
	return inst
end

local function Insanity(inst,eater)
	if eater and PotionManager then
		PotionManager:Track(eater,{name="Insanity",strength=4,dur=48},inst)
	else
		print("WARNING! Failed to apply insanity effect!")
	end
end

local function fn2()
	local inst = common()
	
	inst.AnimState:PlayAnimation("stew")
	
	inst.components.edible.healthvalue = TUNING.HEALING_MEDSMALL
	inst.components.edible.hungervalue = TUNING.CALORIES_MEDSMALL
	inst.components.edible.sanityvalue = TUNING.SANITY_HUGE
	inst.components.edible.foodtype = "MEAT"
	
	inst.components.edible.oneaten = Insanity
	
	inst.components.perishable:SetPerishTime(TUNING.PERISH_SUPERFAST)
	inst.components.perishable:StartPerishing()
	
	inst:AddTag("meat")
	
	return inst
end

return Prefab("common/inventory/soulcake", fn1, assets, prefabs),
	Prefab("common/inventory/soulstew", fn2, assets, prefabs)