require "brains/abigailbrain"
require "stategraphs/SGghost"

local assets = {
	Asset("ANIM", "anim/ghost.zip"),
	Asset("ANIM", "anim/ghost_build.zip"),
	Asset("SOUND", "sound/ghost.fsb"),
}

local function IsHostile(inst,target) --true if either of us targets this or if it targets us
	local player = inst.components.follower.leader
	return (inst.components.combat.target and inst.components.combat.target == target)
	or (player and player.components.combat and player.components.combat.target
	and player.components.combat.target == target)
	or (target.components.combat and target.components.combat.target
	and (target.components.combat.target == inst or target.components.combat.target == player))
end

local function Retarget(inst)
	local newtarget = FindEntity(inst, 20, function(guy)
			return guy.components.combat and inst.components.combat:CanTarget(guy)
			and IsHostile(inst,guy)
		end)

	return newtarget
end

local function BreakArmor(inst)
	if inst.armor and inst.armor.components.armor then
		inst.armor.components.armor:SetPercent(inst.armor.components.armor:GetPercent()-.4)
	end
end

local function Rebind(inst,data)
	if inst.armor and inst.armor.components.armor then
		inst.armor.components.armor:SetCondition(inst.armor.components.armor.condition - 20)
		inst.armor.guard = nil
	end
	inst.sg:GoToState("dissipate")
	inst:DoTaskInTime(1,function() inst:Remove() end)
end

local function fn(Sim)
	local inst = CreateEntity()
	inst.entity:AddTransform()
	inst.entity:AddAnimState()
	inst.entity:AddSoundEmitter()
	inst.entity:AddPhysics()
	
	MakeGhostPhysics(inst, 1, .5)
	
	local light = inst.entity:AddLight()
	light:SetIntensity(.6)
	light:SetRadius(.5)
	light:SetFalloff(.6)
	light:Enable(true)
	light:SetColour(180/255, 195/255, 225/255)
	
	inst.persists = false --respawned by armour
	
	local brain = require "brains/abigailbrain"
	inst:SetBrain(brain)
	
	inst.AnimState:SetBank("ghost")
	inst.AnimState:SetBuild("ghost_build")
	inst.AnimState:PlayAnimation("idle", true)
	inst.AnimState:SetMultColour(.9,1,.85,1)
	inst.AnimState:SetBloomEffectHandle( "shaders/anim.ksh" )
	
	inst:AddComponent("locomotor")
	inst.components.locomotor.walkspeed = TUNING.ABIGAIL_SPEED
	inst.components.locomotor.runspeed = TUNING.ABIGAIL_SPEED
	inst.components.locomotor.directdrive = true --?
	
	inst:SetStateGraph("SGghost")
	
	--inst:AddComponent("sanityaura")
	--inst.components.sanityaura.aura = -TUNING.SANITYAURA_MED
	
	inst:AddComponent("inspectable")
	
	inst:AddComponent("health")
	inst.components.health:SetMaxHealth(TUNING.ABIGAIL_HEALTH /2)
	inst.components.health:StartRegen(1,1)
	
	inst:AddComponent("combat")
	inst.components.combat.defaultdamage = TUNING.GHOST_DAMAGE
	inst.components.combat:SetRetargetFunction(3, Retarget)
	
	inst:AddComponent("follower")
	
	inst:AddComponent("aura") --there is aura damage!
	inst.components.aura.radius = TUNING.GHOST_RADIUS
	inst.components.aura.tickperiod = TUNING.GHOST_DMG_PERIOD
	inst.components.aura.auratestfn = IsHostile
	
	inst.SoundEmitter:PlaySound("dontstarve/ghost/ghost_howl_LP", "howl")
	
	inst:ListenForEvent("death",BreakArmor)
	inst:ListenForEvent("loseloyalty",Rebind)
	inst:ListenForEvent("stopfollowing",Rebind)
	
	inst:AddTag("ghost")
	inst:AddTag("noauradamage")
	inst:AddTag("notraptrigger")
	inst:AddTag("scarytoprey")
	
	return inst
end

return Prefab("common/monsters/soulguard", fn, assets) 
