local assets = {
	Asset("ANIM", "anim/hat_soul.zip"),
	Asset("ANIM", "anim/hat_ruins.zip"),
}

local function Repair(inst)
	local guy = inst.components.inventoryitem.owner
	if guy and guy.components.health and not (guy.components.health.currenthealth < 4) then
		local needs = inst.components.armor.maxcondition - inst.components.armor.condition
		local val = math.min(3,needs,guy.components.health.currenthealth)
		
		if val > 0 then
			inst.components.armor:SetCondition(inst.components.armor.condition + val)
			guy.components.health:DoDelta(-val,true,inst.prefab)
		end
	end
end

local function onequip(inst, owner) 
	owner.AnimState:OverrideSymbol("swap_hat", "hat_soul", "swap_hat")
	owner.AnimState:Show("HAT")
	owner.AnimState:Hide("HAT_HAIR")
	owner.AnimState:Show("HAIR_NOHAT")
	owner.AnimState:Show("HAIR")
	
	owner.AnimState:Show("HEAD")
	owner.AnimState:Hide("HEAD_HAIR")
	
	inst.AutoRepair = inst:DoPeriodicTask(1.5,Repair)
end

local function onunequip(inst, owner)
	owner.AnimState:Hide("HAT")
	owner.AnimState:Hide("HAT_HAIR")
	owner.AnimState:Show("HAIR_NOHAT")
	owner.AnimState:Show("HAIR")

	if owner:HasTag("player") then
		owner.AnimState:Show("HEAD")
		owner.AnimState:Hide("HEAD_HAIR")
	end
	
	if inst.AutoRepair then
		inst.AutoRepair:Cancel()
		inst.AutoRepair = nil
	end
end


local function fn()
	local inst = CreateEntity()
	inst.entity:AddTransform()
	inst.entity:AddAnimState()
	MakeInventoryPhysics(inst)
	
	inst.AnimState:SetBank("ruinshat")
	inst.AnimState:SetBuild("hat_soul")
	inst.AnimState:PlayAnimation("anim")
	
	inst:AddComponent("inspectable")
	inst:AddComponent("inventoryitem")
	inst.components.inventoryitem.atlasname = "images/soul/soulhat.xml"
	
	inst:AddComponent("tradable")
	
	inst:AddComponent("equippable")
	inst.components.equippable.equipslot = EQUIPSLOTS.HEAD
	inst.components.equippable:SetOnEquip( onequip )
	inst.components.equippable:SetOnUnequip( onunequip )
	
	inst:AddComponent("armor")
	inst.components.armor:InitCondition(TUNING.ARMOR_FOOTBALLHAT, TUNING.FULL_ABSORPTION * .99)
	inst.components.armor.onfinished = function()
		if inst.AutoRepair then
			inst.AutoRepair:Cancel()
			inst.AutoRepair = nil
		end
	end
	
	inst:AddTag("hat")
	inst:AddTag("sanity")
	inst:AddTag("soul")

	return inst
end

return Prefab("common/inventory/soulhat",fn,assets,prefabs)