local assets = {
	Asset("ANIM", "anim/spellcandle.zip"),
}

local NEIGHBOR_DIST = 4 --within this radius, candles are considered next to each other

local function ondeploy(inst,pt)
	inst.Physics:Teleport(pt:Get())
	inst.components.inventoryitem.canbepickedup = false
	inst.components.activatable.inactive = true
	inst:DoTaskInTime(5,function()
		if not inst.flickertask then --hack
			inst.components.inventoryitem.canbepickedup = true
			inst.components.activatable.inactive = false
		end
	end)
end

local function ResetCandles(inst)
	inst.circlecenter = nil
	inst.radius = nil
	for k,v in pairs(inst.candles) do
		v:RemoveTag("hasbeenvisited")
		v.components.activatable.inactive = false
		v.components.inventoryitem.canbepickedup = true
		v:GoOut()
	end
end

local function GetNeighbors(inst,census,countlist)
	local pos = inst:GetPosition()
	local count = 0
	
	inst:AddTag("hasbeenvisited") --to prevent loops
	
	--saPrint("visiting",inst)
	
	for guid,ent in pairs(TheSim:FindEntities(pos.x,pos.y,pos.z,NEIGHBOR_DIST,{"nidus"},{"INLIMBO"})) do
		if ent ~= inst then
			count = count + 1
			if not ent:HasTag("hasbeenvisited") then
				ent:GetNeighbors(census,countlist)
			end
		end
	end
	
	table.insert(census,inst)
	--we only want to know if there's such a case
	countlist[count] = true
end

local function Flicker(inst)
    inst.flickertask = nil
    inst.AnimState:PlayAnimation("flicker")
	inst.AnimState:PushAnimation("burn",true)
    inst.flickertask = inst:DoTaskInTime(.5+math.random(),Flicker)
end

local function Kindle(inst)
	--play a calm flame animation, every now and then flicker
	inst.AnimState:PlayAnimation("ignite")
	inst.AnimState:PushAnimation("burn",true)
	--the next bit is inspired by gold's shine effect
	inst:DoTaskInTime(.5+math.random(),Flicker)
end

local function GoOut(inst)
	if inst.flickertask then
		inst.flickertask:Cancel()
		inst.flickertask = nil
	end
	inst.AnimState:PlayAnimation("out")
	inst.AnimState:PushAnimation("idle",false)
end

local function Activate(inst,guy)
	saPrint("ACTIVATED A SPELL CANDLE---------------------------------")
	
	--check for surrounding candles
	inst.candles = {}
	local neighborcounts = {}
	
	inst:GetNeighbors(inst.candles,neighborcounts)
	
	--does every candle have exactly two neighbors?
	local few = false
	local many = false
	for k,v in pairs(neighborcounts) do
		if k < 2 then
			few = true
		elseif
			k > 2 then
			many = true
		end
	end
	
	local valid = true
	
	if few and many then --wtf is that shape, a line?!
		print("invalid shape") --regular prints, so you know what you're doing wrong :P
		if guy.components.talker then
			guy.components.talker:Say(GetString(guy.prefab,"ANNOUNCE_SPELLCIRCLE","INVALID_SHAPE"))
		end
		valid = false
	elseif few then --if too few, say the circle is too weak
		print("too far appart")
		if guy.components.talker then
			guy.components.talker:Say(GetString(guy.prefab,"ANNOUNCE_SPELLCIRCLE","SMALL"))
		end
		valid = false
	elseif many then --if too many, say the circle is too tight
		print("too tightly placed")
		if guy.components.talker then
			guy.components.talker:Say(GetString(guy.prefab,"ANNOUNCE_SPELLCIRCLE","BIG"))
		end
		valid = false
	elseif #inst.candles < 3 then --at least four candles
		print("too few candles")
		if guy.components.talker then
			guy.components.talker:Say(GetString(guy.prefab,"ANNOUNCE_SPELLCIRCLE","SMALL"))
		end
		valid = false
	elseif #inst.candles > 7 then --at most six (to avoid super-range spells)
		print("too many candles")
		if guy.components.talker then
			guy.components.talker:Say(GetString(guy.prefab,"ANNOUNCE_SPELLCIRCLE","BIG"))
		end
		valid = false
	end
	
	if valid then
		--saPrint("valid")
		--calculate the center of the circle using math!
		--saPrint("getting the center")
		local sum = Vector3(0,0,0) --one can add vectors together
		for k,v in pairs(inst.candles) do --includes this inst
			--saPrint("adding pos of",inst)
			sum = sum + v:GetPosition()
		end
		inst.circlecenter = sum / #inst.candles
		
		--calculate the radius of the circle using similiar math
		--saPrint("getting the radius")
		sum = 0
		for k,v in pairs(inst.candles) do --includes this inst
			--saPrint("adding dist of",inst)
			sum = sum + v:GetDistanceSqToPoint(inst.circlecenter)
		end
		inst.radius = math.sqrt( sum / #inst.candles )
		
		--check whether a spell, and only one, is possible
		--unsuprisingly, this is a bit intensive
		saPrint("CALCULATING POSSIBLE SPELLS")
		local spellid --the spells numeric key
		local catalysts = {} --the actual catalysts used in the spell
		local ingredients = {} --the actual ingredients consumed in the spell
		
		local ingr_a = {} --temporary catalysts
		local ingr_b = {} --temporary ingredients
		local num_a = {} --because I'm lazy
		local num_b = {} --also because it's probably cleaner
		local ents = TheSim:FindEntities(inst.circlecenter.x,inst.circlecenter.y,inst.circlecenter.z,inst.radius,nil,{"INLIMBO"})
		for k,v in pairs(NidusSpells) do
			saPrint("Checking spell",v.name)
			--gather data on the requirements
			for l,w in pairs(v.catalysts) do --required count
				num_a[l] = w.num or 1
			end
			for l,w in pairs(v.ingredients) do
				num_b[l] = w.num or 1
			end
			
			for guid,ent in pairs(ents) do
				saPrint("checking new entity:",ent)
				if ent and ent:IsValid() then
					
					if v.catalysts then
						for l,w in pairs(v.catalysts) do
							if type(w) ~= "table" then print("INVALID CATALYST!",w) else
								
								local tags_ok = true
								if w.tags then --check the tag requirements
									for _,tag in pairs(w.tags) do
										if not ent:HasTag(tag) then
											tags_ok = false --all tags must match
										end
									end
								end
								
								if (not w.prefab or ent.prefab == w.prefab) --check prefab
								and (not w.fn or w.fn(ent)) --and fn
								and tags_ok and num_a[l] then --and tags and remaining
									saPrint("confirmed as catalyst",l)
									if not ingr_a[l] then
										ingr_a[l] = {} --table for several of the same type
									end
									
									if ent.components.stackable then --likely in stack form
										local new_ent = ent.components.stackable:Get(num_a[l]) --returns inst if less
										num_a[l] = num_a[l] - ent.components.stackable:StackSize()
										
										table.insert(ingr_a[l],new_ent)
										if new_ent ~= ent then
											new_ent.Transform:SetPosition(ent.Transform:GetWorldPosition())
										end
									else
										num_a[l] = num_a[l] -1 --reduce counter
										table.insert(ingr_a[l],ent)
									end
									if num_a[l] < 1 then --is that ingredient still needed
										num_a[l] = false
									end
								end
							end
						end
					end
					
					if v.ingredients then
						for l,w in pairs(v.ingredients) do
							if type(w) ~= "table" then print("INVALID INGREDIENT!",w) else
								
								local tags_ok = true
								if w.tags then
									for tag,_ in pairs(w.tags) do
										if not ent:HasTag(tag) then
											tags_ok = false
										end
									end
								end
								
								if (not w.prefab or ent.prefab == w.prefab)
								and (not w.fn or w.fn(ent))
								and tags_ok and num_b[l] then
									saPrint("confirmed as ingredient",l)
									if not ingr_b[l] then
										ingr_b[l] = {}
									end
									
									if ent.components.stackable then --likely in stack form
										local new_ent = ent.components.stackable:Get(num_b[l])
										num_b[l] = num_b[l] - ent.components.stackable:StackSize()
										
										table.insert(ingr_b[l],new_ent)
										if new_ent ~= ent then
											new_ent.Transform:SetPosition(ent.Transform:GetWorldPosition())
										end
									else
										num_b[l] = num_b[l] -1 --reduce counter
										table.insert(ingr_b[l],ent)
									end
									if num_b[l] < 1 then --is that ingredient still needed
										num_b[l] = false
									end
								end
							end
						end
					end
				end
			end
			
			--inspect the data
			local passed = true
			for l,w in pairs(v.catalysts) do
				if ingr_a[l] then --that's a table
					if num_a[l] then --gotta check the count afterwards
						passed = false
						saPrint("couldn't match catalyst",l,"reason: count",num_a[l])
					end
				else
					saPrint("couldn't match catalyst",l,"reason: not present")
					passed = false
				end
			end
			for l,w in pairs(v.ingredients) do
				if ingr_b[l] then
					if num_b[l] then
						passed = false
						saPrint("couldn't match ingredient",l,"reason: count",num_b[l])
					end
				else
					passed = false
					saPrint("couldn't match ingredient",l,"reason: not present")
				end
			end
			
			if passed then
				if spellid then --we already had one
					saPrint("ANOTHER SPELL PASSED - aborting")
					spellid = false --lets pretend we didn't have any
					break
				else --success!
					saPrint("SPELL PASSED")
					spellid = k
					catalysts = ingr_a --keep these in mind, they're important
					ingredients = ingr_b
				end
			end
			ingr_a = {} --reset this
			ingr_b = {}
			num_a = {}
			num_b = {}
		end
		
		if spellid then
			saPrint("STARTING RITUAL")
			
			if NidusSpells[spellid].initcb then --instant callback, also test
				local b = NidusSpells[spellid].initcb(guy,catalysts,inst.circlecenter,inst.radius,inst)
				if b == false then
					ResetCandles(inst)
					return false
				end
			end
			
			--prepare the candles
			for k,v in pairs(inst.candles) do --includes this inst
				v:Kindle()
				v.components.inventoryitem.canbepickedup = false
			end
			
			--freeze the ingredients
			for l,w in pairs(ingredients) do
				for k,v in pairs(w) do
					saPrint("freezing",v)
					v:AddTag("notarget")
					v:RemoveComponent("inventoryitem")
					v:RemoveComponent("inspectable")
					v:RemoveComponent("activatable")
					v:RemoveComponent("useableitem")
					v:RemoveComponent("pickable")
					v:RemoveComponent("propagator")
					v:RemoveComponent("explosive")
					if v.components.burnable then
						if v.components.burnable.task then
							v.components.burnable.task:Cancel()
							v.components.burnable.task = nil
						end
						v.components.burnable.burning = true --hopefully goofs it
					end
					if v.components.fueled then
						v.components.fueled:StopConsuming()
					end
					if v.components.health then
						v.components.health.nofadeout = true
					end
					
					--change colour gradually
					local tick_time = TheSim:GetTickTime()
					v:StartThread(function()
						local ticks = 0
						while ticks * tick_time < 2.5 do
							local amount = ticks * tick_time / 2.5
							v.AnimState:SetAddColour(amount,amount,amount,0)
							ticks = ticks + 1
							Yield()
						end
					end)
				end
			end
			
			--freeze catalysts gently
			for l,w in pairs(catalysts) do
				for k,v in pairs(w) do
					saPrint("securing",v)
					if not v:HasTag("notarget") then
						v:AddTag("notarget")
						v:AddTag("NIDUSnotarget")
					end
					if v.components.health and not v.components.health.nofadeout then
						v.components.health.nofadeout = true
						v:AddTag("NIDUSnofadeout")
					end
					if v.components.workable and not v.components.workable.destroyed then
						v.NIDUSworkleft = v.components.workable.workleft
						v.components.workable.workleft = 0
					end
					if v.components.inventoryitem and v.components.health.canbepickedup then
						v.components.inventoryitem.canbepickedup = false
						v:AddTag("NIDUScanbepickedup")
					end
					if v.components.locomotor then
						v.components.locomotor:Stop()
						if v.components.locomotor.enablegroundspeedmultiplier then
							v.components.locomotor:EnableGroundSpeedMultiplier(false)
							v:AddTag("NIDUSenablegsm")
						end
						v.components.locomotor.groundspeedmultiplier = 0
					elseif v.Physics then
						v.Physics:Stop()
					end
					if v.components.burnable and v.components.burnable.task then
						v.components.burnable.task:Cancel()
					end
				end
			end
			
			--runs parallel to other code
			inst:StartThread(function()
				Sleep(2.5) --wait a few seconds
				
				saPrint("RITUAL CONTINUES")
				
				--consume the ingredients
				for l,w in pairs(ingredients) do
					for k,v in pairs(w) do
						saPrint("consuming",v)
						SpawnPrefab("small_puff").Transform:SetPosition(v.Transform:GetWorldPosition())
						v:Remove()
					end
				end
				
				--execute the spell
				if NidusSpells[spellid].cb then
					saPrint("Callback executing...")
					NidusSpells[spellid].cb(guy,catalysts,inst.circlecenter,inst.radius,inst)
				end
				
				--revert the candles to their normal state (anim,tags,pickup)
				ResetCandles(inst)
				
				--revert the catalysts to be free again (if they still exist)
				for _,w in pairs(catalysts) do
					for _,v in pairs(w) do
						saPrint("freeing",v)
						if v and v:IsValid() then
							if v:HasTag("NIDUSnotarget") then
								v:RemoveTag("NIDUSnotarget")
								v:RemoveTag("notarget")
							end
							if v:HasTag("NIDUSnofadeout") and v.components.health then
								v.components.health.nofadeout = false
								v:RemoveTag("NIDUSnofadeout")
							end
							if v:HasTag("NIDUScanbepickedup") and v.components.inventoryitem then
								v.components.inventoryitem.canbepickedup = true
								v:RemoveTag("NIDUScanbepickedup")
							end
							if v:HasTag("NIDUSenablegsm") and v.components.locomotor then
								v.components.locomotor:EnableGroundSpeedMultiplier(true)
							end
							if v.NIDUSworkleft and v.components.workable
							and v.components.workable.workleft == 0 then
								v.components.workable.workleft = v.NIDUSworkleft
							end
						end
					end
				end
			end)
			
			return true
		else
			if guy.components.talker then
				guy.components.talker:Say(GetString(guy.prefab,"ANNOUNCE_SPELLCIRCLE","INVALID_SPELL"))
			end
		end
	end
	--let the player try again
	saPrint("Resetting...")
	ResetCandles(inst)
end

local function fn()
	local inst = CreateEntity()
	inst.entity:AddTransform()
	inst.entity:AddAnimState()
	inst.entity:AddSoundEmitter()
	
	MakeInventoryPhysics(inst)
	
	inst.AnimState:SetBank("spellcandle")
	inst.AnimState:SetBuild("spellcandle")
	inst.AnimState:PlayAnimation("idle")
	
	inst:AddComponent("inspectable")
	
	inst:AddComponent("inventoryitem")
	inst.components.inventoryitem.atlasname = "images/soul/spellcandle.xml"
	
	inst:AddComponent("activatable")
	inst.components.activatable.inactive = false
	inst.components.activatable.OnActivate = Activate
	--inst.components.activatable.quickaction = false
	
	inst:AddComponent("deployable") --for activating
	inst.components.deployable.ondeploy = ondeploy
	
	inst.GetNeighbors = GetNeighbors
	inst.Kindle = Kindle
	inst.GoOut = GoOut
	
	inst:AddTag("candle")
	inst:AddTag("nidus")
	
	return inst
end

return Prefab("common/inventory/spellcandle",fn,assets),
	MakePlacer("common/spellcandle_placer","spellcandle","spellcandle","idle")