local assets = {
	Asset("ANIM", "anim/fishingrod.zip"),
	Asset("ANIM", "anim/starfisher.zip"),
	Asset("ANIM", "anim/swap_starfisher.zip"),
}

prefabs = {
	"soulitem",
	"soulitem_giant"
}

local function onfinished(inst)
	inst:Remove()
end

local function onequip(inst, owner) 
	owner.AnimState:OverrideSymbol("swap_object", "swap_starfisher", "swap_spear")
	owner.AnimState:Show("ARM_carry") 
	owner.AnimState:Hide("ARM_normal") 
end

local function onunequip(inst, owner) 
	owner.AnimState:Hide("ARM_carry") 
	owner.AnimState:Show("ARM_normal") 
end

local function TrySoul(inst,wielder,target)
	if target.components.health and target.components.health:IsDead() then
		--better chance when killed directly
		if math.random() < .4 then
			local name = "soulitem"
			
			if target:HasTag("epic") then
				name = "soulitem_giant"
			elseif IsSoulless(target) then
				return
			end
			
			SpawnPrefab(name).Transform:SetPosition(target.Transform:GetWorldPosition())
		end
	else
		PotionManager:Track(target,{name="Soulbite",strength=1,dur=5,args={nofx=true}},wielder)
	end
end


local function fn()
	local inst = CreateEntity()
	local trans = inst.entity:AddTransform()
	local anim = inst.entity:AddAnimState()
	MakeInventoryPhysics(inst)
	
	inst.AnimState:SetBank("fishingrod")
	inst.AnimState:SetBuild("starfisher")
	inst.AnimState:PlayAnimation("idle")
	--inst.AnimState:SetMultColour(1,1,1,.6)
	
	inst:AddComponent("weapon")
	inst.components.weapon:SetDamage(TUNING.SPEAR_DAMAGE *.9)
	inst.components.weapon:SetOnAttack(TrySoul)
	
	inst:AddComponent("finiteuses")
	inst.components.finiteuses:SetMaxUses(TUNING.SPIKE_USES)
	inst.components.finiteuses:SetUses(TUNING.SPIKE_USES)
	inst.components.finiteuses:SetOnFinished( onfinished )

	inst:AddComponent("inspectable")
	inst:AddComponent("inventoryitem")
	inst.components.inventoryitem.atlasname = "images/soul/starfisher.xml"
	
	inst:AddComponent("equippable")
	inst.components.equippable:SetOnEquip( onequip )
	inst.components.equippable:SetOnUnequip( onunequip )
	
	inst:AddTag("soul")

	return inst
end

return Prefab("common/inventory/starfisher",fn,assets,prefabs)