local Screen = require "widgets/screen"
local Button = require "widgets/button"
local ImageButton = require "widgets/imagebutton"
local Menu = require "widgets/menu"
local Text = require "widgets/text"
local TextEdit = require "widgets/textedit"
local Image = require "widgets/image"
local UIAnim = require "widgets/uianim"
local Widget = require "widgets/widget"
local PopupScreen = require "screens/bigpopupdialog"

local function txtposx(txt)
	local size = txt.string:len() * 7.5 --assuming this is HALF the average character width
	return size - 280 --position left-anchored
end

local TheScreen = Class(Screen, function(self,inst,limited)
	Screen._ctor(self, "PotionReader_Screen")
	
	--self.src = inst
	self.potion = inst.components.container.slots[1]
	--self.player = p
	
	if not (inst and self.potion) then
		TheFrontEnd:PopScreen(self)
		return
	end
	
	self.active = true
	SetPause(true,"pause")
	
	--darken everything behind the dialog
	self.black = self:AddChild(Image("images/global.xml", "square.tex"))
	self.black:SetVRegPoint(ANCHOR_MIDDLE)
	self.black:SetHRegPoint(ANCHOR_MIDDLE)
	self.black:SetVAnchor(ANCHOR_MIDDLE)
	self.black:SetHAnchor(ANCHOR_MIDDLE)
	self.black:SetScaleMode(SCALEMODE_FILLSCREEN)
	self.black:SetTint(0,0,0,.75)	

	self.proot = self:AddChild(Widget("ROOT"))
	self.proot:SetVAnchor(ANCHOR_MIDDLE)
	self.proot:SetHAnchor(ANCHOR_MIDDLE)
	self.proot:SetPosition(0,0,0)
	self.proot:SetScaleMode(SCALEMODE_PROPORTIONAL)

	--display the background
	self.bg = self.proot:AddChild(Image("images/globalpanels.xml", "panel.tex"))
	self.bg:SetVRegPoint(ANCHOR_MIDDLE)
	self.bg:SetHRegPoint(ANCHOR_MIDDLE)
	self.bg:SetScale(1.75,1.5,1)
	
	--data
	self.i = 0 --page index
	
	self.image = self.proot:AddChild(Image())
	self.image:SetPosition(230,190,0)
	
	self.title = self.proot:AddChild(Text(TITLEFONT, 50))
	self.title:SetPosition(-70,190,0)
	
	self.text = self.proot:AddChild(Text(DIALOGFONT, 40))
	self.text:SetPosition(0,0,0)
	
	--two arrow buttons to scroll through
	self.goleft = self.proot:AddChild(ImageButton(nil,"next_arrow.tex","next_arrow_over.tex","next_arrow.tex"))
	self.goleft:SetPosition(-40, -250, 0)
	self.goleft:SetScale(-.7,.7,.7)
	self.goleft:SetOnClick(function()
		self.i = self.i - 1
		self:UpdatePanels()
	end)
	
	self.goright = self.proot:AddChild(ImageButton(nil,"next_arrow.tex","next_arrow_over.tex","next_arrow.tex"))
	self.goright:SetPosition(40, -250, 0)
	self.goright:SetScale(.7,.7,.7)
	self.goright:SetOnClick(function()
		if not limited then
			self.i = self.i + 1
			self:UpdatePanels()
		end
	end)
	
	--a button to rename
	self.rename = self.proot:AddChild(ImageButton())
	self.rename:SetPosition(-180, -250, 0)
	self.rename:SetText("Rename")
	self.rename:SetOnClick(function()
		self:Rename()
	end)
	
	--the resume button
	self.resume = self.proot:AddChild(ImageButton())
	self.resume:SetPosition(180, -250, 0)
	self.resume:SetText(STRINGS.UI.OPTIONS.CLOSE)
	--self.resume:SetFont(BUTTONFONT)
	self.resume:SetOnClick(function()
		TheFrontEnd:PopScreen(self)
		if not self.was_paused then
			SetPause(false)
		end
		GetWorld():PushEvent("continuefrompause")
	end)
	
	self:UpdatePanels()
	
	TheInputProxy:SetCursorVisible(true)
	self.default_focus = self.resume
end)

function TheScreen:Rename()
	print("SCREEN| Rename button pressed")
	
	self.proot:Hide()
	self.pproot = self:AddChild(Widget("ROOT")) --clearly, I have no idea what the p is for
	self.pproot:SetVAnchor(ANCHOR_MIDDLE)
	self.pproot:SetHAnchor(ANCHOR_MIDDLE)
	self.pproot:SetPosition(0,0,0)
	self.pproot:SetScaleMode(SCALEMODE_PROPORTIONAL)
	
	--display the background
	self.subbg = self.pproot:AddChild(Image("images/globalpanels.xml", "small_dialog.tex"))
	self.subbg:SetVRegPoint(ANCHOR_MIDDLE)
	self.subbg:SetHRegPoint(ANCHOR_MIDDLE)
	self.subbg:SetScale(1.5,1.2,1.2)
	
	--help
	self.subtitle = self.pproot:AddChild(Text(DIALOGFONT,30,"Hover and type, confirm with Enter. Leave blank to reset."))
	self.subtitle:SetPosition(0, 55, 0)
	
	--textedit bg
	self.textbg = self.pproot:AddChild(Image("images/ui.xml","textbox_long.tex"))
	self.textbg:SetPosition(-80, -50, 0)
	self.textbg:SetScale(.6,.8,.8)
	
	--actual text input
	self.textedit = self.pproot:AddChild(TextEdit(DIALOGFONT,45,""))
	self.textedit:SetPosition(-80, -50, 0)
	self.textedit.limit = 15
	self.textedit.validchars = [[ abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789.,@!'*+-/=?{}"]]
	self.textedit.OnTextEntered = function(text)
		if text == "" then
			self.potion.forcedname = false
			local name
			for _,eff in pairs(self.potion.fx) do
				if not name or eff.strength > name.strength then
					name = eff
				end
			end
			if name then
				self.potion.components.named:SetName(STRINGS.POTION_NAMES[name.name])
			else
				self.potion.components.named:SetName()
			end
		else
			self.potion.components.named:SetName(text)
			self.potion.forcedname = true
		end
		self.pproot:Kill()
		self.proot:Show()
	end
	self.textedit:SetFocusedImage(self.textbg,"images/ui.xml","textbox_long_over.tex","textbox_long.tex")
	self.textedit:SetRegionSize(480,80)
	
	--return button
	self.goback = self.pproot:AddChild(ImageButton())
	self.goback:SetPosition(240, -45, 0)
	self.goback:SetText(STRINGS.UI.MODSSCREEN.BACKTITLE)
	self.goback:SetOnClick(function()
		self.pproot:Kill()
		self.proot:Show()
	end)
end

function TheScreen:UpdatePanels()	 
	 --update arrows
	 if self.i < 1 then
		self.i = 0
		self.goright:Enable()
		self.goleft:Disable()
	elseif self.i >= #self.potion.fx then
		self.i = #self.potion.fx
		self.goright:Disable()
		self.goleft:Enable()
	else
		self.goright:Enable()
		self.goleft:Enable()
	end
	
	--update text
	if self.i == 0 then --general data
		self.title:SetString("Basic Data")
		
		local isnamed = self.potion.forcedname and "Yes" or "No"
		local numtags = 0
		if self.potion.alchemytags then
			for tag, weight in pairs(self.potion.alchemytags) do
				numtags = numtags + math.min(1,weight) --some tags may be 0
			end
		end
		if numtags == 0 then numtags = "None" end
		
		self.text:SetString("Remaining Uses: ".. self.potion.remaininguses
		.."\nEffects: ".. #self.potion.fx
		.."\nPreviously renamed: ".. isnamed
		.."\nSynthesis Likelyhood: ".. numtags)
		
		self.image:SetTexture("images/alchemypotion.xml",self.potion.anim and self.potion.anim ..".tex" or "new.tex")
		
	else --effect data
		local eff = self.potion.fx[self.i]
		self.title:SetString("Effect: ".. eff.name)
		
		local numanti = 0
		if eff.anti then
			for tag, _ in pairs(eff.anti) do
				numanti = numanti + 1
			end
		end
		if numanti == 0 then numanti = "None" end
		local numalt = 0
		if eff.alt then
			for tag, _ in pairs(eff.alt) do
				numalt = numalt + 1
			end
		end
		if numalt == 0 then numalt = "None" end
		local duration = eff.dur or "Instant"
		
		self.text:SetString("Strength: ".. eff.strength
		.."\nDuration: ".. duration
		.."\nSolutions: ".. numanti
		.."\nReactivity: ".. numalt)
		
		self.image:SetTexture(eff.atlas or "images/potioneffects/".. string.lower(eff.name) ..".xml",eff.image or string.lower(eff.name) ..".tex")
	end
end

function TheScreen:OnControl(control, down) --this is for ESCAPEing the screen :P
	if TheScreen._base.OnControl(self,control, down) then return true end

	if (control == CONTROL_PAUSE or control == CONTROL_CANCEL) and not down then	
		self.active = false
		TheFrontEnd:PopScreen() 
		SetPause(false)
		GetWorld():PushEvent("continuefrompause")
		return true
	end

end

function TheScreen:OnUpdate(dt)
	if self.active then
		SetPause(true)
	end
end

function TheScreen:OnBecomeActive()
	TheScreen._base.OnBecomeActive(self)
	TheFrontEnd:HideTopFade() --not sure what topfade is, possibly "top" as in "z layer"?
end

return TheScreen