require("stategraphs/commonstates")

local actionhandlers = 
{
    ActionHandler(ACTIONS.GOHOME, "flyaway"),
}

local events=
{
    EventHandler("locomote", function(inst)
		local is_busy = inst.sg:HasStateTag("busy")
		local is_idling = inst.sg:HasStateTag("idle")
		local is_moving = inst.sg:HasStateTag("moving")
		local is_running = inst.sg:HasStateTag("running")
		local is_sidemoving = inst.sg:HasStateTag("sidemoving")

		if is_busy then return end
		--if is_sidemoving then return end
        if not is_idling and not is_moving and not is_sidemoving then return end
		
		local should_move = inst.components.locomotor:WantsToMoveForward()
		local should_run = inst.components.locomotor:WantsToRun()
		
        if is_moving and not should_move then
			if is_running then
				inst.sg:GoToState("run_stop")
			elseif not is_idling then
                inst.components.locomotor:StopMoving()
				inst.sg:GoToState("idle")
			end
		elseif (not is_moving and should_move) or (is_moving and should_move and is_running ~= should_run) then
			if should_run then
				inst.sg:GoToState("run_start")
			else
				inst.sg:GoToState("move")
			end
		end
    end),
	
    EventHandler("movetoside", function(inst)
		if inst.sg:HasStateTag("busy") or inst.sg:HasStateTag("sidemoving") then return end
		
		inst.sg:GoToState("move_side")
		
		if inst.components.combat.target then --Hack
			inst:FacePoint(inst.components.combat.target.Transform:GetWorldPosition())
		end
	end),
	
    EventHandler("shoot", function(inst)
		if inst.sg:HasStateTag("busy") then return end
		
		inst.sg:GoToState("shoot")
	end),
	
    --CommonHandlers.OnAttacked(),
	EventHandler("attacked", function(inst)
        if inst.components.health and not inst.components.health:IsDead()
           and inst.sg:HasStateTag("frozen") then
            inst.sg:GoToState("idle")
        end
    end),
	
    CommonHandlers.OnFreeze(),
    CommonHandlers.OnDeath(),

    EventHandler("flyaway", function(inst) 
        if not inst.components.health:IsDead() then -- and not inst.sg:HasStateTag("busy")
            inst.sg:GoToState("flyaway") 
        end 
    end),
}


local states=
{
    State{        
        name = "idle",
        tags = {"idle", "canrotate"},
        
        onenter = function(inst, data)
            inst.Physics:Stop()
			inst.AnimState:PushAnimation("idle_loop", true)
        end,
      
        timeline = {},
    },
	
	State{
		name = "move",
		tags = {"moving", "canrotate"},
		
		onenter = function(inst)
			if inst.components.locomotor then
				inst.components.locomotor:RunForward()
			end
			inst.AnimState:PushAnimation("idle_loop")
		end,
		
		events=
		{   
			EventHandler("animqueueover", function(inst) inst.sg:GoToState("move") end ),        
		},
	},
	
	State{
		name = "move_side",
		tags = {"sidemoving", "canrotate"},
		
		onenter = function(inst) 
			if inst.components.locomotor then
				inst.components.locomotor:Clear()
				inst.Physics:SetMotorVel(inst.components.locomotor:GetWalkSpeed()*.1,0,inst.components.locomotor:GetWalkSpeed())
			end
			
			inst.AnimState:PushAnimation("idle_loop")
		end,
		
		events=
		{   
			EventHandler("animqueueover", function(inst) inst.sg:GoToState("move_side") end ),        
		},
	},
	
	State{
		name = "shoot",
		tags = {"sidemoving", "busy", "attack", "canrotate"},
		
		onenter = function(inst)
			--inst.Physics:Stop()
			if inst.components.locomotor then
				inst.components.locomotor:Stop() --for after catching up
				inst.Physics:SetMotorVel(inst.components.locomotor:GetWalkSpeed()*.1,0,inst.components.locomotor:GetWalkSpeed())
			end
			
			inst.AnimState:PlayAnimation("shoot")
		end,

        timeline =
        {
            TimeEvent(9*FRAMES, function(inst) 
				--inst.SoundEmitter:PlaySound("dontstarve/creatures/krampus/kick_whoosh")
				inst.SoundEmitter:PlaySound("dontstarve/frog/attack_spit")
				local shot = SpawnPrefab("runicprojectile")
				shot.components.weapon:SetDamage(CFG_SA.ELDEREYE_SHOTDAMAGE)
				shot.Transform:SetPosition(inst.Transform:GetWorldPosition())
				shot.components.projectile:Throw(inst, inst.components.combat.target, inst)
            end),
        },
		
		events=
		{   
			EventHandler("animover", function(inst)
				inst:OnShoot()
				inst.sg:GoToState("move_side")
			end ),
		},
	},
	
	State{
		name = "run_start",
		tags = {"moving", "running", "busy", "atk_pre", "canrotate"},
		
		onenter = function(inst)
			inst.Physics:Stop()
			inst.AnimState:PlayAnimation("atk_pre")
		end,
		
		events=
		{
			EventHandler("animover", function(inst)
				inst.SoundEmitter:PlaySound("dontstarve/common/ghost_spawn")
				inst.sg:GoToState("run")
			end),
		}
	},

    State{
		name = "run",
		tags = {"moving", "running", "ramming"},
		
		onenter = function(inst)
			if inst.components.locomotor then
				inst.components.locomotor:RunForward()
			end
			
			inst.AnimState:PlayAnimation("atk")
		end,
		
		events=
		{   
			EventHandler("animover", function(inst) inst.sg:GoToState("run") end ),        
		},
	},
    
    State{
		name = "run_stop",
		tags = {"atk_pst"}, --"idle", 
		
		onenter = function(inst)
			if inst.components.locomotor then
				inst.components.locomotor:Stop()
			end
			
			inst.AnimState:PlayAnimation("atk_pst")
			
			--hurt target if we haven't already (or trigger miss event)
			if not (inst.components.combat.target and inst.recentlycharged
			and inst.recentlycharged[inst.components.combat.target.GUID]) then
				inst.components.combat:DoAttack()
			end
		end,
		
		events =
		{
			EventHandler("animover", function(inst)
				inst.sg:AddStateTag("canrotate")
				inst:OnRam()
				inst.sg:GoToState("idle")
			end),
		},
	},
	
	--[[State{
        name = "hit",
        tags = {"hit", "busy"},
        
        onenter = function(inst, cb)
            --inst.components.locomotor:StopMoving()
            inst.AnimState:PlayAnimation("hurt")
        end,
        
        events =
        {
            EventHandler("animover", function(inst) inst.sg:GoToState("idle") end),
        },
    }]]
	
   
    State{
        name = "flyaway",
        tags = {"flying", "busy"},

        onenter = function(inst)
            inst.Physics:Stop()
			inst.components.health:SetInvincible(true)
			
			inst.AnimState:PlayAnimation("flee")
			inst:StopMusic()
        end,
		
        timeline =
        {
            TimeEvent(51*FRAMES, function(inst)
				inst.SoundEmitter:PlaySound("dontstarve/creatures/krampus/bag_jumpinto")
			end),
        },
		
        events=
        {
            EventHandler("animover", function(inst)
			    inst.DynamicShadow:Enable(false)
                inst.Physics:SetMotorVel(0,math.random(5,7),0) 
				inst:DoTaskInTime(4,inst.Remove)
			end),
        },
    },
	
	State{
        name = "death",
        tags = {"busy"},
        
        onenter = function(inst)
            --inst.components.locomotor:Stop()
			--inst.components.locomotor:SetBufferedAction()
			--inst.components.locomotor:ResetPath()
			--inst.components.locomotor:Clear()
            inst.Physics:Stop()
			inst.Physics:ClearCollisionMask()
			inst:AddTag("NOCLICK")
			
            inst.AnimState:PlayAnimation("death")
			inst.SoundEmitter:PlaySound("dontstarve/common/throne/thronedisappear")
			--"dontstarve/creatures/rook_minotaur/death_voice"
			inst:StopMusic()
			
			if PotionManager then
				if not PotionManager.arbitrarydata then
					PotionManager.arbitrarydata = {}
				end
				PotionManager.arbitrarydata.killedeldereye = true
			end
        end,
		
        timeline =
        {
            TimeEvent(10*FRAMES, function(inst) inst.components.lootdropper:SpawnLootPrefab(weighted_random_choice(inst.loots)) end),
            TimeEvent(20*FRAMES, function(inst) inst.components.lootdropper:SpawnLootPrefab(weighted_random_choice(inst.loots)) end),
            TimeEvent(30*FRAMES, function(inst) inst.components.lootdropper:SpawnLootPrefab(weighted_random_choice(inst.loots)) end),
            TimeEvent(40*FRAMES, function(inst) inst.components.lootdropper:SpawnLootPrefab(weighted_random_choice(inst.loots)) end),
            TimeEvent(50*FRAMES, function(inst) inst.components.lootdropper:SpawnLootPrefab(weighted_random_choice(inst.loots)) end),
            TimeEvent(60*FRAMES, function(inst) inst.components.lootdropper:SpawnLootPrefab(weighted_random_choice(inst.loots)) end),
            TimeEvent(70*FRAMES, function(inst) inst.components.lootdropper:SpawnLootPrefab(weighted_random_choice(inst.loots)) end),
            TimeEvent(80*FRAMES, function(inst) inst.components.lootdropper:SpawnLootPrefab(weighted_random_choice(inst.loots)) end),
            --TimeEvent(90*FRAMES, function(inst) end), --stop dying
            TimeEvent(103*FRAMES, function(inst)
				inst.SoundEmitter:PlaySound("dontstarve/creatures/krampus/bag_jumpinto")
			end),
        },
		
        events=
        {
            EventHandler("animover", function(inst)
				inst.SoundEmitter:PlaySound("dontstarve/creatures/krampus/bag_jumpinto")
			    inst.DynamicShadow:Enable(false)
				inst.components.lootdropper:SetLoot(inst.goodloots)
				inst.components.lootdropper:DropLoot()
				inst:RemoveComponent("locomotor") -- So radical!
				inst.Physics:SetMotorVel(0,math.random(5,7),0) 
				inst:DoTaskInTime(4,inst.Remove)
				inst.persists = false
			end),
        },
    }
}

CommonStates.AddFrozenStates(states)

return StateGraph("eldereye", states, events, "idle", actionhandlers)