local require = GLOBAL.require
require "saveindex"
--local SaveGameIndex = GLOBAL.SaveGameIndex
local Ents = GLOBAL.Ents
local TheSim = GLOBAL.TheSim
local SpawnPrefab = GLOBAL.SpawnPrefab
local Vector3 = GLOBAL.Vector3
--local PotionManager = GLOBAL.PotionManager --can't refer to it yet
local TUNING = GLOBAL.TUNING
local PI = GLOBAL.PI
local GROUND = GLOBAL.GROUND

local function GetStatShuffle()
	return math.random() * .9 + .1
end

local function Leifable(inst)
	return inst and inst.prefab and inst.prefab:sub(1,9) == "evergreen"
		and not (inst:HasTag("burnt") or inst:HasTag("stump") or inst.noleif == true)
		and (not inst.components.growable or inst.components.growable.stage <= 3)
end

local function MakeLeif(inst)
	if inst and Leifable(inst) then
		local name = "leif"
		if inst.prefab:sub(10,16) == "_sparse" then
			name = "leif_sparse"
		end
		--the following is a crude copy of the code in evergreen.lua
		inst.noleif = true
		inst.leifscale = .5 + inst.components.growable.stage * .25
		inst:DoTaskInTime(1 + math.random()*3, function() 
			if inst and not inst:HasTag("stump") and not inst:HasTag("burnt") and
				inst.components.growable and inst.components.growable.stage <= 3 then
				local leif = SpawnPrefab(name)
				if leif then
					local scale = inst.leifscale
					local r,g,b,a = inst.AnimState:GetMultColour()
					leif.AnimState:SetMultColour(r,g,b,a)
					
					leif.components.locomotor.walkspeed = leif.components.locomotor.walkspeed*scale
					leif.components.combat.defaultdamage = leif.components.combat.defaultdamage*scale
					leif.components.health.maxhealth = leif.components.health.maxhealth*scale
					leif.components.health.currenthealth = leif.components.health.currenthealth*scale
					leif.components.combat.hitrange = leif.components.combat.hitrange*scale
					leif.components.combat.attackrange = leif.components.combat.attackrange*scale
					
					leif.Transform:SetScale(scale,scale,scale) 
					leif.components.combat:SuggestTarget(caster)
					leif.sg:GoToState("spawn")
					leif.Transform:SetPosition(inst.Transform:GetWorldPosition())
					inst:Remove()
				end
			end
		end)
	end
end

local function MassSpawn(prefab,num,center,radius)
	--make sure we have all necessary data
	if not (prefab and type(prefab) == "string" and GLOBAL.PrefabExists(prefab)
	and center and type(center) == "table" and center.x and center.z ) then
		print("CRITICAL ERROR in MassSpawn",prefab,center)
		return {}
	end
	radius = radius or 8
	num = num or 12
	
	--make the entities
	local ents = {}
	--random approach
	--[[for i = 1,num do
		local offset = FindWalkableOffset(center,math.random()*2*PI,radius*math.random())
		if offset then
			table.insert(ents,SpawnPrefab(prefab))
			ents[#ents].Transform:SetPosition((center + offset):Get())
		end
	end]]
	--ordered approach
	local lim = math.floor(num*.45)
	radius = radius - 1
	for i = 1, lim do
		local angle = i/lim *2*PI
		local pt = Vector3(math.sin(angle)*radius,0,math.cos(angle)*radius) + center
		local tile = GLOBAL.GetWorld().Map:GetTileAtPoint(pt.x, pt.y, pt.z)
		if tile ~= GROUND.IMPASSABLE and tile < GROUND.UNDERGROUND then
			table.insert(ents,SpawnPrefab(prefab))
			ents[#ents].Transform:SetPosition(pt.x,pt.y,pt.z)
			--TODO optional visual effects
		end
	end
	local lim = math.ceil(num*.55)
	radius = radius + 2
	for i = 1, lim do
		--TODO calculate better rotation-offset?
		local angle = (i/lim + .3) *2*PI
		local pt = Vector3(math.sin(angle)*radius,0,math.cos(angle)*radius) + center
		local tile = GLOBAL.GetWorld().Map:GetTileAtPoint(pt.x, pt.y, pt.z)
		if tile ~= GROUND.IMPASSABLE and tile < GROUND.UNDERGROUND then
			table.insert(ents,SpawnPrefab(prefab))
			ents[#ents].Transform:SetPosition(pt.x,pt.y,pt.z)
		end
	end
	
	return ents
end

--you can insert your own spells using
--table.insert(GLOBAL.NidusSpells,MySpellTable)

GLOBAL.NidusSpells = {

{ --works
	name = "unseeing",
	catalysts = {{tags={"player"}}},
	ingredients = {{prefab="silk",num=4},{prefab="soulitem_giant"}},
	--initcb = function(caster, catalysts, center, radius, initcandle) end,
	cb = function(caster, catalysts, center, radius, initcandle)
		for guid, ent in pairs(TheSim:FindEntities(center.x,center.y,center.z,radius*2,nil,{"INLIMBO"})) do
			GLOBAL.PotionManager:Track(ent,{name="Invisible",strength=6,dur=40})
			GLOBAL.PotionManager:Track(ent,{name="Blind",strength=7,dur=40,args={alpha=.95}})
		end
		--prepare eldereye spawn
		GLOBAL.PotionManager:Track(caster,{
			name="Summon", strength=18,
			dur=400+math.random()*300,
			args={prefab="eldereyeinit", delayed=true}
		})
	end,
},
{ --works
	name = "ghosts",
	catalysts = {{prefab="gravestone"}},
	ingredients = {{prefab="livinglog"},{prefab="meat",num=2}},
	--initcb = function(caster, catalysts, center, radius, initcandle) end,
	cb = function(caster, catalysts, center, radius, initcandle)
		MassSpawn("ghost",8,center,radius)
	end,
},
{ --works
	name = "mermen",
	catalysts = {{fn=function(inst)
		return inst.prefab and inst.prefab:sub(1,4) == "pond"
	end}},
	ingredients = {{prefab="bedroll_straw"},{fn=function(inst)
		if inst.prefab == "alchemypotion" and inst.fx then
			for k,v in pairs(inst.fx) do
				if v.name == "Sleep" then
					return true
				end
			end
		end
	end}},
	--initcb = function(caster, catalysts, center, radius, initcandle) end,
	cb = function(caster, catalysts, center, radius, initcandle)
		for k,v in pairs(MassSpawn("merm",10,center,radius)) do
			if v.components.sleeper then
				v.components.sleeper:GoToSleep(9)
			end
		end
	end,
},
{ --works
	name = "leif",
	catalysts = {{fn = Leifable}},
	ingredients = {{prefab="axe"},{prefab="nightmarefuel"}},
	--initcb = function(caster, catalysts, center, radius, initcandle) end,
	cb = function(caster, catalysts, center, radius, initcandle)
		MakeLeif(catalysts[1][1])
	end,
},
{ --works
	name = "occult",
	catalysts = {{fn=function(inst)
		return inst.prefab and inst.prefab:sub(1,8) == "skeleton"
	end}},
	ingredients = {{prefab="pumpkin_lantern"}},
	--initcb = function(caster, catalysts, center, radius, initcandle) end,
	cb = function(caster, catalysts, center, radius, initcandle)
		for guid, ent in pairs(TheSim:FindEntities(center.x,center.y,center.z,radius*2,nil,{"INLIMBO"})) do
			if ent.prefab == "abigail_flower" then
				local new = SpawnPrefab("abigail")
				new.Transform:SetPosition(ent.Transform:GetWorldPosition())
				ent.SoundEmitter:PlaySound("dontstarve/common/ghost_spawn")
				ent:Remove()
			elseif ent.components.werebeast then
				ent.components.werebeast:SetWere(30) --TUNE were time
			elseif ent.prefab == "mound" and ent.components.childspawner then
				ent.components.childspawner:ReleaseAllChildren() --only works in RoG
			elseif ent:HasTag("tree") and not ent:HasTag("fire") then
				if math.random() < .3 then
					if Leifable(ent) then
						MakeLeif(ent)
					end
				else
					if ent.prefab ~= "livingtree" and not ent:HasTag("magic") then
						local new = SpawnPrefab("livingtree")
						new.Transform:SetPosition(ent.Transform:GetWorldPosition())
						--TODO needs visual fx for transition
						ent:Remove()
					end
				end
			end
		end

	end,
},
{ --works
	name = "soulbite",
	catalysts = {},
	ingredients = {{prefab="pitchfork"},{prefab="dug_grass"},{prefab="fish"}},
	--initcb = function(caster, catalysts, center, radius, initcandle) end,
	cb = function(caster, catalysts, center, radius, initcandle)
		for guid, ent in pairs(TheSim:FindEntities(center.x,center.y,center.z,radius*2,nil,{"INLIMBO"})) do
			GLOBAL.PotionManager:Track(ent,{name="Soulbite",strength=3,dur=18,args={chance=1}})
		end
	end,
},
{ --works
	name = "defense",
	catalysts = {},
	ingredients = {{prefab="armormarble"},{fn=function(inst)
		if inst.prefab == "alchemypotion" and inst.fx then
			for k,v in pairs(inst.fx) do
				if v.name == "Shield" then
					return true
				end
			end
		end
	end}},
	--initcb = function(caster, catalysts, center, radius, initcandle) end,
	cb = function(caster, catalysts, center, radius, initcandle)
		for guid, ent in pairs(TheSim:FindEntities(center.x,center.y,center.z,radius,nil,{"INLIMBO"})) do
			GLOBAL.PotionManager:Track(ent,{name="Shield",strength=9,args={val=.75,shields=5},dur=38})
			GLOBAL.PotionManager:Track(ent,{name="GrueImmune",strength=0,dur=28})
			GLOBAL.PotionManager:Track(ent,{name="IceImmune",strength=1,dur=26})
			GLOBAL.PotionManager:Track(ent,{name="FireImmune",strength=1,dur=27})
		end
	end,
},
{ --works
	name = "peace",
	catalysts = {},
	ingredients = {{prefab="charcoal"},{prefab="flint"},{prefab="cutreeds"}},
	--initcb = function(caster, catalysts, center, radius, initcandle) end,
	cb = function(caster, catalysts, center, radius, initcandle)
		for guid, ent in pairs(TheSim:FindEntities(center.x,center.y,center.z,radius*3,nil,{"INLIMBO"})) do
			if ent.components.combat then
				ent.components.combat:GiveUp()
				GLOBAL.PotionManager:Track(ent,{name="Invisible",strength=3,dur=23,args={visible=true}})
			end
		end
	end,
},
{ --works
	name = "insanity",
	catalysts = {},
	ingredients = {{prefab="petals_evil",num=2},{prefab="smallmeat"},{prefab="campfire"}},
	--initcb = function(caster, catalysts, center, radius, initcandle) end,
	cb = function(caster, catalysts, center, radius, initcandle)
		for guid, ent in pairs(TheSim:FindEntities(center.x,center.y,center.z,radius*2)) do
			if ent.components.sanity then
				ent.components.sanity:SetPercent(0)
			end
		end
	end,
},
{ --works
	name = "massmurder",
	catalysts = {{tags={"player"}}},
	ingredients = {{prefab="monstermeat",num=10},{fn=function(inst)
		if inst.prefab == "alchemypotion" and inst.fx then
			for k,v in pairs(inst.fx) do
				if v.name == "Damage" or v.name == "Poison" or v.name == "Acid" then
					return true
				end
			end
		end
	end}},
	--initcb = function(caster, catalysts, center, radius, initcandle) end,
	cb = function(caster, catalysts, center, radius, initcandle)
		for guid, ent in pairs(TheSim:FindEntities(center.x,center.y,center.z,radius*2,nil,{"INLIMBO"})) do
			if ent.components.health then
				ent.components.health:Kill()
			end
		end
	end,
},
{ --stable, mostly works
	name = "statshuffle",
	catalysts = {},
	ingredients = {{prefab="rocks",num=3},{prefab="compass"},{prefab="froglegs"}},
	--initcb = function(caster, catalysts, center, radius, initcandle) end,
	cb = function(caster, catalysts, center, radius, initcandle)
		for guid, ent in pairs(TheSim:FindEntities(center.x,center.y,center.z,radius*2)) do
			if ent.components.sanity then --works
				ent.components.sanity:SetPercent(GetStatShuffle())
			end
			if ent.components.health then --works
				ent.components.health:SetPercent(GetStatShuffle())
			end
			if ent.components.hunger then --works
				ent.components.hunger:SetPercent(GetStatShuffle())
			end
			if ent.components.armor then --works
				ent.components.armor:SetPercent(GetStatShuffle())
			end
			if ent.components.perishable then --works
				ent.components.perishable:SetPercent(GetStatShuffle())
			end
			if ent.components.fueled then --works
				ent.components.fueled:SetPercent(GetStatShuffle())
			end
			if ent.components.finiteuses then --works
				ent.components.finiteuses:SetPercent(GetStatShuffle())
			end
			--workable?
			--growable?
			if ent.components.crop and not ent.components.crop.matured then --untested
				ent.components.crop.growthpercent = math.random()
				ent.AnimState:SetPercent("grow", ent.components.crop.growthpercent)
			end
			if ent.components.temperature then --works
				local val = math.random(ent.components.temperature.mintemp,ent.components.temperature.maxtemp)
				ent.components.temperature:SetTemperature(val) --TODO make gentler?
			end
			if ent.components.machine then --untested
				if math.random < .5 then
					if ent.components.machine:IsOn() then
						ent.components.machine:TurnOff()
					end
				else
					if not ent.components.machine:IsOn() then
						ent.components.machine:TurnOn()
					end
				end
			end
			--cancelled due to mad hacking: beard
		end
	end,
},
{ --works
	name = "tallbird",
	catalysts = {},
	ingredients = {{prefab="crow"},{prefab="beefalowool",num=15},{prefab="tallbirdegg"},
	{fn=function(inst)
		if inst.prefab == "alchemypotion" and inst.fx then
			for k,v in pairs(inst.fx) do
				if v.name == "RunSpeed" then
					return true
				end
			end
		end
	end}},
	--initcb = function(caster, catalysts, center, radius, initcandle) end,
	cb = function(caster, catalysts, center, radius, initcandle)
		SpawnPrefab("small_puff").Transform:SetPosition(center.x,center.y,center.z)
		SpawnPrefab("tallbirdnest").Transform:SetPosition(center.x,center.y,center.z)
	end,
},
{ --works
	name = "cave",
	catalysts = {},
	ingredients = {{prefab="gunpowder"},{prefab="goldenshovel"},{prefab="rabbit"},
	{prefab="red_cap",num=5},{prefab="green_cap",num=5},{prefab="blue_cap",num=5},
	{fn=function(inst)
		if inst.prefab == "alchemypotion" and inst.fx then
			for k,v in pairs(inst.fx) do
				if v.name == "Acid" then
					return true
				end
			end
		end
	end}},
	initcb = function(caster, catalysts, center, radius, initcandle)
		local world = GLOBAL.GetWorld() --or GLOBAL.TheWorld
		if not (world and GLOBAL.SaveGameIndex:GetCurrentMode() == "survival") then
			return false
		end
		
		local count = 0
		for k,v in pairs(Ents) do
			if v.prefab and v.prefab == "cave_entrance" then
				count = count +1
			end
		end
		
		return count < 6
	end,
	cb = function(caster, catalysts, center, radius, initcandle)
		SpawnPrefab("small_puff").Transform:SetPosition(center.x,center.y,center.z)
		SpawnPrefab("cave_entrance").Transform:SetPosition(center.x,center.y,center.z)
	end,
},
{ --works
	name = "rain",
	catalysts = {},
	ingredients = {{prefab="honey",num=3},{prefab="papyrus"}},
	--initcb = function(caster, catalysts, center, radius, initcandle) end,
	cb = function(caster, catalysts, center, radius, initcandle)
		local sm = GLOBAL.GetSeasonManager() --or GLOBAL.TheWorld
		if sm then
			if sm.precip then
				sm.atmo_moisture = sm.atmo_moisture + (sm.moisture_limit/2)
			else
				sm:StartPrecip()
			end
		end
		if caster and caster.components.sanity then
			caster.components.sanity:DoDelta(TUNING.SANITY_MED)
		end
	end,
},
{ --fix harvest funciton
	name = "picky",
	catalysts = {},
	ingredients = {{prefab="goldnugget"},{prefab="pomegranate"},{prefab="houndstooth"}},
	--initcb = function(caster, catalysts, center, radius, initcandle) end,
	cb = function(caster, catalysts, center, radius, initcandle)
		for guid, ent in pairs(TheSim:FindEntities(center.x,center.y,center.z,radius*3,nil,{"INLIMBO"})) do
		
			if ent.components.pickable then
				if ent.components.pickable.product then
					loot = SpawnPrefab(ent.components.pickable.product)
					if loot then
						if ent.components.pickable.numtoharvest > 1 and loot.components.stackable then
							loot.components.stackable:SetStackSize(ent.components.pickable.numtoharvest)
						end
						loot.Transform:SetPosition(center.x,center.y,center.z)
					end
				end
				ent.components.pickable:Pick(initcandle)
			end
			
			if ent.components.harvest then
				if ent.components.harvest.product then
					loot = SpawnPrefab(ent.components.harvest.product)
					if loot then
						if ent.components.harvest.produce > 1 and loot.components.stackable then
							loot.components.stackable:SetStackSize(ent.components.harvest.produce)
						end
						loot.Transform:SetPosition(center.x,center.y,center.z)
					end
				end
				ent.components.harvest:Harvest(initcandle)
			end
			
			if ent.components.inventoryitem and ent.components.inventoryitem.canbepickedup then
				ent.Transform:SetPosition(center.x,center.y,center.z)
			end
					
		end
	end,
},

--[[{ --DEBUG SPELL
	name = "debug",
	catalysts = {{tags={"player"}}},
	ingredients = {{prefab="starpotion"}},
	--initcb = function(caster, catalysts, center, radius, initcandle) end,
	--cb = function(caster, catalysts, center, radius, initcandle) end,
},]]

}
